@@ -67,6 +67,7 @@ use crate::dbg::DisplayList;
 use crate::dominator_tree::DominatorTree;
 use crate::entity::SparseSet;
 use crate::flowgraph::{BlockPredecessor, ControlFlowGraph};
+use crate::ir::ExceptionTableItem;
 use crate::ir::entities::AnyEntity;
 use crate::ir::instructions::{CallInfo, InstructionFormat, ResolvedConstraint};
 use crate::ir::{self, ArgumentExtension, BlockArg, ExceptionTable};
@@ -75,8 +76,7 @@ use crate::ir::{
     JumpTable, MemFlags, MemoryTypeData, Opcode, SigRef, StackSlot, Type, Value, ValueDef,
     ValueList, types,
 };
-use crate::ir::{ExceptionTableItem, Signature};
-use crate::isa::{CallConv, TargetIsa};
+use crate::isa::TargetIsa;
 use crate::print_errors::pretty_verifier_error;
 use crate::settings::FlagsOrIsa;
 use crate::timing;
@@ -99,7 +99,7 @@ pub struct VerifierError {
 
 // This is manually implementing Error and Display instead of using thiserror to reduce the amount
 // of dependencies used by Cranelift.
-impl std::error::Error for VerifierError {}
+impl core::error::Error for VerifierError {}
 
 impl Display for VerifierError {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
@@ -178,7 +178,7 @@ pub struct VerifierErrors(pub Vec<VerifierError>);
 
 // This is manually implementing Error and Display instead of using thiserror to reduce the amount
 // of dependencies used by Cranelift.
-impl std::error::Error for VerifierErrors {}
+impl core::error::Error for VerifierErrors {}
 
 impl VerifierErrors {
     /// Return a new `VerifierErrors` struct.
@@ -594,7 +594,7 @@ impl<'a> Verifier<'a> {
             } => {
                 self.verify_func_ref(inst, func_ref, errors)?;
                 self.verify_value_list(inst, args, errors)?;
-                self.verify_callee_abi(inst, func_ref, opcode, errors)?;
+                self.verify_callee_patchability(inst, func_ref, opcode, errors)?;
             }
             CallIndirect {
                 sig_ref, ref args, ..
@@ -955,34 +955,40 @@ impl<'a> Verifier<'a> {
         Ok(())
     }
 
-    fn verify_callee_abi(
+    fn verify_callee_patchability(
         &self,
         inst: Inst,
         func_ref: FuncRef,
         opcode: Opcode,
         errors: &mut VerifierErrors,
     ) -> VerifierStepResult {
-        let callee_sig_ref = self.func.dfg.ext_funcs[func_ref].signature;
-        let callee_sig = &self.func.dfg.signatures[callee_sig_ref];
-        let callee_abi = callee_sig.call_conv;
-        match (opcode, callee_abi) {
-            (Opcode::PatchableCall, CallConv::Patchable) => {
-                if !self.func.dfg.ext_funcs[func_ref].colocated {
-                    errors.fatal((
-                        inst,
-                        self.context(inst),
-                        "patchable call to non-colocated function".to_string(),
-                    ))?;
-                }
-            }
-            (Opcode::PatchableCall, _) => {
-                errors.fatal((
-                    inst,
-                    self.context(inst),
-                    "patchable call with non-patchable-ABI callee".to_string(),
-                ))?;
-            }
-            _ => {}
+        let ir::ExtFuncData {
+            patchable,
+            colocated,
+            signature,
+            name: _,
+        } = self.func.dfg.ext_funcs[func_ref];
+        let signature = &self.func.dfg.signatures[signature];
+        if patchable && (opcode == Opcode::ReturnCall || opcode == Opcode::ReturnCallIndirect) {
+            errors.fatal((
+                inst,
+                self.context(inst),
+                "patchable funcref cannot be used in a return_call".to_string(),
+            ))?;
+        }
+        if patchable && !colocated {
+            errors.fatal((
+                inst,
+                self.context(inst),
+                "patchable call to non-colocated function".to_string(),
+            ))?;
+        }
+        if patchable && !signature.returns.is_empty() {
+            errors.fatal((
+                inst,
+                self.context(inst),
+                "patchable call cannot occur to a function with return values".to_string(),
+            ))?;
         }
         Ok(())
     }
@@ -2010,8 +2016,6 @@ impl<'a> Verifier<'a> {
             }
         }
 
-        self.verify_signature(AnyEntity::Function, &self.func.signature, errors)?;
-
         if errors.has_error() { Err(()) } else { Ok(()) }
     }
 
@@ -2078,64 +2082,12 @@ impl<'a> Verifier<'a> {
         Ok(())
     }
 
-    fn verify_signature(
-        &self,
-        loc: impl Into<AnyEntity>,
-        data: &Signature,
-        errors: &mut VerifierErrors,
-    ) -> VerifierStepResult {
-        if data.call_conv == CallConv::Patchable {
-            if data.params.len() > 4 {
-                return errors.fatal((
-                    loc,
-                    "signature with patchable ABI does not allow more than four arguments"
-                        .to_string(),
-                ));
-            }
-            if data.returns.len() > 0 {
-                return errors.fatal((
-                    loc,
-                    "signature with patchable ABI does not allow any returns".to_string(),
-                ));
-            }
-            for param in &data.params {
-                if param.value_type != crate::ir::types::I32
-                    && param.value_type != crate::ir::types::I64
-                {
-                    return errors.fatal((
-                        loc,
-                        "signature with patchable ABI does not allow non-I32/I64 arguments"
-                            .to_string(),
-                    ));
-                }
-                if param.extension != ArgumentExtension::None {
-                    return errors.fatal((
-                        loc,
-                        "signature with patchable ABI does not allow sign/zero-extended arguments"
-                            .to_string(),
-                    ));
-                }
-            }
-        }
-        Ok(())
-    }
-
-    fn verify_signatures(&self, errors: &mut VerifierErrors) -> VerifierStepResult {
-        // Check that "patchable" ABI signatures have no returns and
-        // only up to four integer-typed args.
-        for (sigref, data) in &self.func.dfg.signatures {
-            self.verify_signature(sigref, data, errors)?;
-        }
-        Ok(())
-    }
-
     pub fn run(&self, errors: &mut VerifierErrors) -> VerifierStepResult {
         self.verify_global_values(errors)?;
         self.verify_memory_types(errors)?;
         self.typecheck_entry_block_params(errors)?;
         self.check_entry_not_cold(errors)?;
         self.typecheck_function_signature(errors)?;
-        self.verify_signatures(errors)?;
 
         for block in self.func.layout.blocks() {
             if self.func.layout.first_inst(block).is_none() {
