@@ -610,10 +610,7 @@ impl SafepointSpiller {
     /// rewrite the function's instructions to spill and reload them as
     /// necessary.
     pub fn run(&mut self, func: &mut Function, stack_map_values: &EntitySet<ir::Value>) {
-        log::trace!(
-            "values needing inclusion in stack maps: {:?}",
-            stack_map_values
-        );
+        log::trace!("values needing inclusion in stack maps: {stack_map_values:?}");
         log::trace!(
             "before inserting safepoint spills and reloads:\n{}",
             func.display()
@@ -823,6 +820,7 @@ mod tests {
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Here the value `v1` is technically not live but our single-pass liveness
@@ -840,7 +838,7 @@ mod tests {
         builder.declare_value_needs_stack_map(b);
         builder.switch_to_block(block0);
         builder.ins().call(func_ref, &[a]);
-        builder.ins().jump(block0, &[a, b]);
+        builder.ins().jump(block0, &[a.into(), b.into()]);
         builder.seal_all_blocks();
         builder.finalize();
 
@@ -889,6 +887,7 @@ block0(v0: i32, v1: i32):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // At each `call` we are losing one more value as no longer live, so
@@ -981,6 +980,7 @@ block0:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Here we rely on the post-order to make sure that we never visit block
@@ -1083,6 +1083,7 @@ block3:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // We should not have a stack map entry for `v1` in block 1 because it
@@ -1172,6 +1173,7 @@ block2:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let block0 = builder.create_block();
@@ -1241,6 +1243,7 @@ block2:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Depending on which post-order traversal we take, we might consider
@@ -1328,6 +1331,7 @@ block2:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let block0 = builder.create_block();
@@ -1395,6 +1399,7 @@ block2:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Create an if/else CFG diamond that and check that various things get
@@ -1434,7 +1439,7 @@ block2:
         let v2 = builder.ins().iconst(ir::types::I64, 2);
         builder.declare_value_needs_stack_map(v2);
         builder.ins().call(func_ref, &[]);
-        builder.ins().jump(block3, &[v1, v2]);
+        builder.ins().jump(block3, &[v1.into(), v2.into()]);
 
         builder.switch_to_block(block2);
         let v3 = builder.ins().iconst(ir::types::I64, 3);
@@ -1442,7 +1447,7 @@ block2:
         let v4 = builder.ins().iconst(ir::types::I64, 4);
         builder.declare_value_needs_stack_map(v4);
         builder.ins().call(func_ref, &[]);
-        builder.ins().jump(block3, &[v3, v3]);
+        builder.ins().jump(block3, &[v3.into(), v3.into()]);
 
         builder.switch_to_block(block3);
         builder.append_block_param(block3, ir::types::I64);
@@ -1536,6 +1541,7 @@ block3(v5: i64, v6: i64):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Test that we support stack maps of heterogeneous types and properly
@@ -1621,6 +1627,7 @@ block0(v0: i8, v1: i16, v2: i32, v3: i64, v4: i128, v5: f32, v6: f64, v7: i8x16,
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let name = builder
@@ -1636,6 +1643,7 @@ block0(v0: i8, v1: i16, v2: i32, v3: i64, v4: i128, v5: f32, v6: f64, v7: i8x16,
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Create a series of needs-stack-map values that do not have
@@ -1741,6 +1749,7 @@ block0:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Use a variable, create a control flow diamond so that the variable
@@ -1767,8 +1776,7 @@ block0:
         //                          call $foo(x)
         //                          return x
 
-        let x = Variable::from_u32(0);
-        builder.declare_var(x, ir::types::I32);
+        let x = builder.declare_var(ir::types::I32);
         builder.declare_var_needs_stack_map(x);
 
         let block0 = builder.create_block();
@@ -1884,8 +1892,7 @@ block3(v6: i32):
         let mut func = Function::with_name_signature(ir::UserFuncName::testcase("sample"), sig);
         let mut builder = FunctionBuilder::new(&mut func, &mut fn_ctx);
 
-        let var = Variable::from_u32(0);
-        builder.declare_var(var, cranelift_codegen::ir::types::I32);
+        let var = builder.declare_var(cranelift_codegen::ir::types::I32);
         builder.declare_var_needs_stack_map(var);
 
         let name = builder
@@ -1901,6 +1908,7 @@ block3(v6: i32):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let block0 = builder.create_block();
@@ -1963,6 +1971,7 @@ block0(v0: i32):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Regression test found via fuzzing in
@@ -2040,6 +2049,7 @@ block0(v0: i32):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let name = builder
@@ -2053,6 +2063,7 @@ block0(v0: i32):
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Test that we support stack maps in loops and that we properly handle
@@ -2139,6 +2150,7 @@ block1:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let name = builder
@@ -2154,6 +2166,7 @@ block1:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Test an irreducible loop with multiple entry points, both block1 and
@@ -2278,6 +2291,7 @@ block4:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         let name = builder
@@ -2293,6 +2307,7 @@ block4:
             name: ir::ExternalName::user(name),
             signature,
             colocated: true,
+            patchable: false,
         });
 
         // Test that we detect the `block1 -> block2 -> block3 -> block2 ->
@@ -2338,7 +2353,7 @@ block4:
         builder.declare_value_needs_stack_map(v2);
         let v3 = builder.func.dfg.block_params(block0)[3];
 
-        builder.ins().jump(block1, &[v3]);
+        builder.ins().jump(block1, &[v3.into()]);
 
         builder.switch_to_block(block1);
         let v4 = builder.append_block_param(block1, ir::types::I32);
@@ -2352,7 +2367,7 @@ block4:
         builder.ins().call(bar_func_ref, &[v1]);
         builder.ins().call(foo_func_ref, &[]);
         let v5 = builder.ins().iadd_imm(v4, -1);
-        builder.ins().brif(v4, block1, &[v5], block3, &[]);
+        builder.ins().brif(v4, block1, &[v5.into()], block3, &[]);
 
         builder.switch_to_block(block3);
         builder.ins().call(foo_func_ref, &[]);
@@ -2435,6 +2450,7 @@ block3:
             name,
             signature,
             colocated: true,
+            patchable: false,
         })
     }
 
@@ -2540,12 +2556,10 @@ block3:
         //     block_return:
         //       return
 
-        let var_struct = Variable::from_u32(0);
-        builder.declare_var(var_struct, cranelift_codegen::ir::types::I32);
+        let var_struct = builder.declare_var(cranelift_codegen::ir::types::I32);
         builder.declare_var_needs_stack_map(var_struct);
 
-        let var_array = Variable::from_u32(1);
-        builder.declare_var(var_array, cranelift_codegen::ir::types::I32);
+        let var_array = builder.declare_var(cranelift_codegen::ir::types::I32);
         builder.declare_var_needs_stack_map(var_array);
 
         let block_entry = builder.create_block();
@@ -2573,7 +2587,7 @@ block3:
         let v1 = builder.func.dfg.first_result(call_inst);
         builder.def_var(var_array, v1);
         let v2 = builder.ins().iconst(ir::types::I32, 0);
-        builder.ins().jump(block_array_init_loop_head, &[v2]);
+        builder.ins().jump(block_array_init_loop_head, &[v2.into()]);
 
         builder.switch_to_block(block_array_init_loop_head);
         let v3 = builder.append_block_param(block_array_init_loop_head, ir::types::I32);
@@ -2594,7 +2608,7 @@ block3:
         builder.ins().call(array_init_elem, &[v1, v4]);
         let v6 = builder.ins().iconst(ir::types::I32, 1);
         let v7 = builder.ins().iadd(v4, v6);
-        builder.ins().jump(block_array_init_loop_head, &[v7]);
+        builder.ins().jump(block_array_init_loop_head, &[v7.into()]);
         builder.seal_block(block_array_init_loop_head);
 
         builder.switch_to_block(block_array_init_loop_done);
@@ -2608,7 +2622,7 @@ block3:
         builder.ins().brif(
             v10,
             block_ref_test_done,
-            &[v9],
+            &[v9.into()],
             block_ref_test_non_null,
             &[],
         );
@@ -2620,15 +2634,19 @@ block3:
         let v12 = builder.ins().iconst(ir::types::I32, 0xbeefbeef);
         let v13 = builder.ins().icmp(ir::condcodes::IntCC::Equal, v11, v12);
         let v14 = builder.ins().iconst(ir::types::I32, 1);
-        builder
-            .ins()
-            .brif(v13, block_ref_test_done, &[v14], block_ref_test_slow, &[]);
+        builder.ins().brif(
+            v13,
+            block_ref_test_done,
+            &[v14.into()],
+            block_ref_test_slow,
+            &[],
+        );
 
         builder.switch_to_block(block_ref_test_slow);
         builder.seal_block(block_ref_test_slow);
         let call_inst = builder.ins().call(ref_test, &[v8, v12]);
         let v15 = builder.func.dfg.first_result(call_inst);
-        builder.ins().jump(block_ref_test_done, &[v15]);
+        builder.ins().jump(block_ref_test_done, &[v15.into()]);
 
         builder.switch_to_block(block_ref_test_done);
         let v16 = builder.append_block_param(block_ref_test_done, ir::types::I32);
