@@ -141,9 +141,10 @@ use clap::Parser;
 use std::os::raw::{c_int, c_void};
 use std::slice;
 use std::{env, path::PathBuf};
-use wasi_common::{sync::WasiCtxBuilder, I32Exit, WasiCtx};
 use wasmtime::{Engine, Instance, Linker, Module, Store};
 use wasmtime_cli_flags::CommonOptions;
+use wasmtime_wasi::cli::{InputFile, OutputFile};
+use wasmtime_wasi::{DirPerms, FilePerms, I32Exit, WasiCtx, p1::WasiP1Ctx};
 
 pub type ExitCode = c_int;
 pub const OK: ExitCode = 0;
@@ -272,15 +273,6 @@ pub extern "C" fn wasm_bench_create(
 ) -> ExitCode {
     let result = (|| -> Result<_> {
         let working_dir = config.working_dir()?;
-        let working_dir =
-            cap_std::fs::Dir::open_ambient_dir(&working_dir, cap_std::ambient_authority())
-                .with_context(|| {
-                    format!(
-                        "failed to preopen the working directory: {}",
-                        working_dir.display(),
-                    )
-                })?;
-
         let stdout_path = config.stdout_path()?;
         let stderr_path = config.stderr_path()?;
         let stdin_path = config.stdin_path()?;
@@ -298,39 +290,33 @@ pub extern "C" fn wasm_bench_create(
             config.execution_start,
             config.execution_end,
             move || {
-                let mut cx = WasiCtxBuilder::new();
+                let mut cx = WasiCtx::builder();
 
                 let stdout = std::fs::File::create(&stdout_path)
                     .with_context(|| format!("failed to create {}", stdout_path.display()))?;
-                let stdout = cap_std::fs::File::from_std(stdout);
-                let stdout = wasi_common::sync::file::File::from_cap_std(stdout);
-                cx.stdout(Box::new(stdout));
+                cx.stdout(OutputFile::new(stdout));
 
                 let stderr = std::fs::File::create(&stderr_path)
                     .with_context(|| format!("failed to create {}", stderr_path.display()))?;
-                let stderr = cap_std::fs::File::from_std(stderr);
-                let stderr = wasi_common::sync::file::File::from_cap_std(stderr);
-                cx.stderr(Box::new(stderr));
+                cx.stderr(OutputFile::new(stderr));
 
                 if let Some(stdin_path) = &stdin_path {
                     let stdin = std::fs::File::open(stdin_path)
                         .with_context(|| format!("failed to open {}", stdin_path.display()))?;
-                    let stdin = cap_std::fs::File::from_std(stdin);
-                    let stdin = wasi_common::sync::file::File::from_cap_std(stdin);
-                    cx.stdin(Box::new(stdin));
+                    cx.stdin(InputFile::new(stdin));
                 }
 
                 // Allow access to the working directory so that the benchmark can read
                 // its input workload(s).
-                cx.preopened_dir(working_dir.try_clone()?, ".")?;
+                cx.preopened_dir(working_dir.clone(), ".", DirPerms::READ, FilePerms::READ)?;
 
                 // Pass this env var along so that the benchmark program can use smaller
                 // input workload(s) if it has them and that has been requested.
                 if let Ok(val) = env::var("WASM_BENCH_USE_SMALL_WORKLOAD") {
-                    cx.env("WASM_BENCH_USE_SMALL_WORKLOAD", &val)?;
+                    cx.env("WASM_BENCH_USE_SMALL_WORKLOAD", &val);
                 }
 
-                Ok(cx.build())
+                Ok(cx.build_p1())
             },
         )?);
         Ok(Box::into_raw(state) as _)
@@ -407,7 +393,7 @@ struct BenchState {
     instantiation_timer: *mut u8,
     instantiation_start: extern "C" fn(*mut u8),
     instantiation_end: extern "C" fn(*mut u8),
-    make_wasi_cx: Box<dyn FnMut() -> Result<WasiCtx>>,
+    make_wasi_cx: Box<dyn FnMut() -> Result<WasiP1Ctx>>,
     module: Option<Module>,
     store_and_instance: Option<(Store<HostState>, Instance)>,
     epoch_interruption: bool,
@@ -415,7 +401,7 @@ struct BenchState {
 }
 
 struct HostState {
-    wasi: WasiCtx,
+    wasi: WasiP1Ctx,
     #[cfg(feature = "wasi-nn")]
     wasi_nn: wasmtime_wasi_nn::witx::WasiNnCtx,
 }
@@ -432,11 +418,11 @@ impl BenchState {
         execution_timer: *mut u8,
         execution_start: extern "C" fn(*mut u8),
         execution_end: extern "C" fn(*mut u8),
-        make_wasi_cx: impl FnMut() -> Result<WasiCtx> + 'static,
+        make_wasi_cx: impl FnMut() -> Result<WasiP1Ctx> + 'static,
     ) -> Result<Self> {
         let mut config = options.config(None)?;
         // NB: always disable the compilation cache.
-        config.disable_cache();
+        config.cache(None);
         let engine = Engine::new(&config)?;
         let mut linker = Linker::<HostState>::new(&engine);
 
@@ -459,7 +445,7 @@ impl BenchState {
         let fuel = options.wasm.fuel;
 
         if options.wasi.common != Some(false) {
-            wasi_common::sync::add_to_linker(&mut linker, |cx| &mut cx.wasi)?;
+            wasmtime_wasi::p1::add_to_linker_sync(&mut linker, |cx| &mut cx.wasi)?;
         }
 
         #[cfg(feature = "wasi-nn")]
