@@ -61,19 +61,6 @@ impl LinkOptions {
         self
     }
 }
-pub enum Baz {}
-pub trait HostBaz: Sized {
-    fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> ();
-    fn drop(&mut self, rep: wasmtime::component::Resource<Baz>) -> wasmtime::Result<()>;
-}
-impl<_T: HostBaz + ?Sized> HostBaz for &mut _T {
-    fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> () {
-        HostBaz::foo(*self, self_)
-    }
-    fn drop(&mut self, rep: wasmtime::component::Resource<Baz>) -> wasmtime::Result<()> {
-        HostBaz::drop(*self, rep)
-    }
-}
 impl core::convert::From<LinkOptions> for foo::foo::the_interface::LinkOptions {
     fn from(src: LinkOptions) -> Self {
         (&src).into()
@@ -91,6 +78,24 @@ impl core::convert::From<&LinkOptions> for foo::foo::the_interface::LinkOptions
         dest
     }
 }
+pub enum Baz {}
+pub trait HostBazWithStore: wasmtime::component::HasData {}
+impl<_T: ?Sized> HostBazWithStore for _T
+where
+    _T: wasmtime::component::HasData,
+{}
+pub trait HostBaz {
+    fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> ();
+    fn drop(&mut self, rep: wasmtime::component::Resource<Baz>) -> wasmtime::Result<()>;
+}
+impl<_T: HostBaz + ?Sized> HostBaz for &mut _T {
+    fn foo(&mut self, self_: wasmtime::component::Resource<Baz>) -> () {
+        HostBaz::foo(*self, self_)
+    }
+    fn drop(&mut self, rep: wasmtime::component::Resource<Baz>) -> wasmtime::Result<()> {
+        HostBaz::drop(*self, rep)
+    }
+}
 /// Auto-generated bindings for a pre-instantiated version of a
 /// component which implements the world `the-world`.
 ///
@@ -99,11 +104,11 @@ impl core::convert::From<&LinkOptions> for foo::foo::the_interface::LinkOptions
 /// has been created through a [`Linker`](wasmtime::component::Linker).
 ///
 /// For more information see [`TheWorld`] as well.
-pub struct TheWorldPre<T> {
+pub struct TheWorldPre<T: 'static> {
     instance_pre: wasmtime::component::InstancePre<T>,
     indices: TheWorldIndices,
 }
-impl<T> Clone for TheWorldPre<T> {
+impl<T: 'static> Clone for TheWorldPre<T> {
     fn clone(&self) -> Self {
         Self {
             instance_pre: self.instance_pre.clone(),
@@ -111,7 +116,7 @@ impl<T> Clone for TheWorldPre<T> {
         }
     }
 }
-impl<_T> TheWorldPre<_T> {
+impl<_T: 'static> TheWorldPre<_T> {
     /// Creates a new copy of `TheWorldPre` bindings which can then
     /// be used to instantiate into a particular store.
     ///
@@ -120,7 +125,7 @@ impl<_T> TheWorldPre<_T> {
     pub fn new(
         instance_pre: wasmtime::component::InstancePre<_T>,
     ) -> wasmtime::Result<Self> {
-        let indices = TheWorldIndices::new(instance_pre.component())?;
+        let indices = TheWorldIndices::new(&instance_pre)?;
         Ok(Self { instance_pre, indices })
     }
     pub fn engine(&self) -> &wasmtime::Engine {
@@ -145,6 +150,17 @@ impl<_T> TheWorldPre<_T> {
         self.indices.load(&mut store, &instance)
     }
 }
+impl<_T: Send + 'static> TheWorldPre<_T> {
+    /// Same as [`Self::instantiate`], except with `async`.
+    pub async fn instantiate_async(
+        &self,
+        mut store: impl wasmtime::AsContextMut<Data = _T>,
+    ) -> wasmtime::Result<TheWorld> {
+        let mut store = store.as_context_mut();
+        let instance = self.instance_pre.instantiate_async(&mut store).await?;
+        self.indices.load(&mut store, &instance)
+    }
+}
 /// Auto-generated bindings for index of the exports of
 /// `the-world`.
 ///
@@ -173,11 +189,6 @@ pub struct TheWorldIndices {}
 /// * If you've instantiated the instance yourself already
 ///   then you can use [`TheWorld::new`].
 ///
-/// * You can also access the guts of instantiation through
-///   [`TheWorldIndices::new_instance`] followed
-///   by [`TheWorldIndices::load`] to crate an instance of this
-///   type.
-///
 /// These methods are all equivalent to one another and move
 /// around the tradeoff of what work is performed when.
 ///
@@ -185,22 +196,14 @@ pub struct TheWorldIndices {}
 /// [`Component`]: wasmtime::component::Component
 /// [`Linker`]: wasmtime::component::Linker
 pub struct TheWorld {}
+pub trait TheWorldImportsWithStore: wasmtime::component::HasData + HostBazWithStore {}
+impl<_T: ?Sized> TheWorldImportsWithStore for _T
+where
+    _T: wasmtime::component::HasData + HostBazWithStore,
+{}
 pub trait TheWorldImports: HostBaz {
     fn foo(&mut self) -> ();
 }
-pub trait TheWorldImportsGetHost<
-    T,
-    D,
->: Fn(T) -> <Self as TheWorldImportsGetHost<T, D>>::Host + Send + Sync + Copy + 'static {
-    type Host: TheWorldImports;
-}
-impl<F, T, D, O> TheWorldImportsGetHost<T, D> for F
-where
-    F: Fn(T) -> O + Send + Sync + Copy + 'static,
-    O: TheWorldImports,
-{
-    type Host = O;
-}
 impl<_T: TheWorldImports + ?Sized> TheWorldImports for &mut _T {
     fn foo(&mut self) -> () {
         TheWorldImports::foo(*self)
@@ -215,24 +218,11 @@ const _: () = {
         ///
         /// This method may fail if the component does not have the
         /// required exports.
-        pub fn new(
-            component: &wasmtime::component::Component,
+        pub fn new<_T>(
+            _instance_pre: &wasmtime::component::InstancePre<_T>,
         ) -> wasmtime::Result<Self> {
-            let _component = component;
-            Ok(TheWorldIndices {})
-        }
-        /// Creates a new instance of [`TheWorldIndices`] from an
-        /// instantiated component.
-        ///
-        /// This method of creating a [`TheWorld`] will perform string
-        /// lookups for all exports when this method is called. This
-        /// will only succeed if the provided instance matches the
-        /// requirements of [`TheWorld`].
-        pub fn new_instance(
-            mut store: impl wasmtime::AsContextMut,
-            instance: &wasmtime::component::Instance,
-        ) -> wasmtime::Result<Self> {
-            let _instance = instance;
+            let _component = _instance_pre.component();
+            let _instance_type = _instance_pre.instance_type();
             Ok(TheWorldIndices {})
         }
         /// Uses the indices stored in `self` to load an instance
@@ -245,6 +235,7 @@ const _: () = {
             mut store: impl wasmtime::AsContextMut,
             instance: &wasmtime::component::Instance,
         ) -> wasmtime::Result<TheWorld> {
+            let _ = &mut store;
             let _instance = instance;
             Ok(TheWorld {})
         }
@@ -253,30 +244,45 @@ const _: () = {
         /// Convenience wrapper around [`TheWorldPre::new`] and
         /// [`TheWorldPre::instantiate`].
         pub fn instantiate<_T>(
-            mut store: impl wasmtime::AsContextMut<Data = _T>,
+            store: impl wasmtime::AsContextMut<Data = _T>,
             component: &wasmtime::component::Component,
             linker: &wasmtime::component::Linker<_T>,
         ) -> wasmtime::Result<TheWorld> {
             let pre = linker.instantiate_pre(component)?;
             TheWorldPre::new(pre)?.instantiate(store)
         }
-        /// Convenience wrapper around [`TheWorldIndices::new_instance`] and
+        /// Convenience wrapper around [`TheWorldIndices::new`] and
         /// [`TheWorldIndices::load`].
         pub fn new(
             mut store: impl wasmtime::AsContextMut,
             instance: &wasmtime::component::Instance,
         ) -> wasmtime::Result<TheWorld> {
-            let indices = TheWorldIndices::new_instance(&mut store, instance)?;
-            indices.load(store, instance)
+            let indices = TheWorldIndices::new(&instance.instance_pre(&store))?;
+            indices.load(&mut store, instance)
         }
-        pub fn add_to_linker_imports_get_host<
-            T,
-            G: for<'a> TheWorldImportsGetHost<&'a mut T, T, Host: TheWorldImports>,
-        >(
+        /// Convenience wrapper around [`TheWorldPre::new`] and
+        /// [`TheWorldPre::instantiate_async`].
+        pub async fn instantiate_async<_T>(
+            store: impl wasmtime::AsContextMut<Data = _T>,
+            component: &wasmtime::component::Component,
+            linker: &wasmtime::component::Linker<_T>,
+        ) -> wasmtime::Result<TheWorld>
+        where
+            _T: Send,
+        {
+            let pre = linker.instantiate_pre(component)?;
+            TheWorldPre::new(pre)?.instantiate_async(store).await
+        }
+        pub fn add_to_linker_imports<T, D>(
             linker: &mut wasmtime::component::Linker<T>,
             options: &LinkOptions,
-            host_getter: G,
-        ) -> wasmtime::Result<()> {
+            host_getter: fn(&mut T) -> D::Data<'_>,
+        ) -> wasmtime::Result<()>
+        where
+            D: TheWorldImportsWithStore,
+            for<'a> D::Data<'a>: TheWorldImports,
+            T: 'static,
+        {
             let mut linker = linker.root();
             if options.experimental_world {
                 if options.experimental_world_resource {
@@ -285,10 +291,8 @@ const _: () = {
                             "baz",
                             wasmtime::component::ResourceType::host::<Baz>(),
                             move |mut store, rep| -> wasmtime::Result<()> {
-                                HostBaz::drop(
-                                    &mut host_getter(store.data_mut()),
-                                    wasmtime::component::Resource::new_own(rep),
-                                )
+                                let resource = wasmtime::component::Resource::new_own(rep);
+                                HostBaz::drop(&mut host_getter(store.data_mut()), resource)
                             },
                         )?;
                 }
@@ -320,22 +324,23 @@ const _: () = {
             }
             Ok(())
         }
-        pub fn add_to_linker<T, U>(
+        pub fn add_to_linker<T, D>(
             linker: &mut wasmtime::component::Linker<T>,
             options: &LinkOptions,
-            get: impl Fn(&mut T) -> &mut U + Send + Sync + Copy + 'static,
+            host_getter: fn(&mut T) -> D::Data<'_>,
         ) -> wasmtime::Result<()>
         where
-            U: foo::foo::the_interface::Host + TheWorldImports,
+            D: foo::foo::the_interface::HostWithStore + TheWorldImportsWithStore,
+            for<'a> D::Data<'a>: foo::foo::the_interface::Host + TheWorldImports,
+            T: 'static,
         {
             if options.experimental_world {
-                Self::add_to_linker_imports_get_host(linker, options, get)?;
+                Self::add_to_linker_imports::<T, D>(linker, options, host_getter)?;
                 if options.experimental_world_interface_import {
-                    foo::foo::the_interface::add_to_linker(
-                        linker,
-                        &options.into(),
-                        get,
-                    )?;
+                    foo::foo::the_interface::add_to_linker::<
+                        T,
+                        D,
+                    >(linker, &options.into(), host_getter)?;
                 }
             }
             Ok(())
@@ -388,7 +393,12 @@ pub mod foo {
                 }
             }
             pub enum Bar {}
-            pub trait HostBar: Sized {
+            pub trait HostBarWithStore: wasmtime::component::HasData {}
+            impl<_T: ?Sized> HostBarWithStore for _T
+            where
+                _T: wasmtime::component::HasData,
+            {}
+            pub trait HostBar {
                 fn foo(&mut self, self_: wasmtime::component::Resource<Bar>) -> ();
                 fn drop(
                     &mut self,
@@ -406,30 +416,29 @@ pub mod foo {
                     HostBar::drop(*self, rep)
                 }
             }
-            pub trait Host: HostBar + Sized {
+            pub trait HostWithStore: wasmtime::component::HasData + HostBarWithStore {}
+            impl<_T: ?Sized> HostWithStore for _T
+            where
+                _T: wasmtime::component::HasData + HostBarWithStore,
+            {}
+            pub trait Host: HostBar {
                 fn foo(&mut self) -> ();
             }
-            pub trait GetHost<
-                T,
-                D,
-            >: Fn(T) -> <Self as GetHost<T, D>>::Host + Send + Sync + Copy + 'static {
-                type Host: Host;
+            impl<_T: Host + ?Sized> Host for &mut _T {
+                fn foo(&mut self) -> () {
+                    Host::foo(*self)
+                }
             }
-            impl<F, T, D, O> GetHost<T, D> for F
-            where
-                F: Fn(T) -> O + Send + Sync + Copy + 'static,
-                O: Host,
-            {
-                type Host = O;
-            }
-            pub fn add_to_linker_get_host<
-                T,
-                G: for<'a> GetHost<&'a mut T, T, Host: Host>,
-            >(
+            pub fn add_to_linker<T, D>(
                 linker: &mut wasmtime::component::Linker<T>,
                 options: &LinkOptions,
-                host_getter: G,
-            ) -> wasmtime::Result<()> {
+                host_getter: fn(&mut T) -> D::Data<'_>,
+            ) -> wasmtime::Result<()>
+            where
+                D: HostWithStore,
+                for<'a> D::Data<'a>: Host,
+                T: 'static,
+            {
                 if options.experimental_interface {
                     let mut inst = linker.instance("foo:foo/the-interface")?;
                     if options.experimental_interface_resource {
@@ -437,10 +446,8 @@ pub mod foo {
                             "bar",
                             wasmtime::component::ResourceType::host::<Bar>(),
                             move |mut store, rep| -> wasmtime::Result<()> {
-                                HostBar::drop(
-                                    &mut host_getter(store.data_mut()),
-                                    wasmtime::component::Resource::new_own(rep),
-                                )
+                                let resource = wasmtime::component::Resource::new_own(rep);
+                                HostBar::drop(&mut host_getter(store.data_mut()), resource)
                             },
                         )?;
                     }
@@ -470,21 +477,6 @@ pub mod foo {
                 }
                 Ok(())
             }
-            pub fn add_to_linker<T, U>(
-                linker: &mut wasmtime::component::Linker<T>,
-                options: &LinkOptions,
-                get: impl Fn(&mut T) -> &mut U + Send + Sync + Copy + 'static,
-            ) -> wasmtime::Result<()>
-            where
-                U: Host,
-            {
-                add_to_linker_get_host(linker, options, get)
-            }
-            impl<_T: Host + ?Sized> Host for &mut _T {
-                fn foo(&mut self) -> () {
-                    Host::foo(*self)
-                }
-            }
         }
     }
 }
