@@ -13,20 +13,18 @@
 //! function body, the imported wasm function do not. The trampolines symbol
 //! names have format "_trampoline_N", where N is `SignatureIndex`.
 
-use crate::{CompiledFunction, RelocationTarget};
-use anyhow::Result;
-use cranelift_codegen::binemit::Reloc;
-use cranelift_codegen::isa::unwind::{systemv, UnwindInfo};
+use crate::CompiledFunction;
 use cranelift_codegen::TextSectionBuilder;
+use cranelift_codegen::isa::unwind::{UnwindInfo, systemv};
 use cranelift_control::ControlPlane;
-use gimli::write::{Address, EhFrame, EndianVec, FrameTable, Writer};
 use gimli::RunTimeEndian;
+use gimli::write::{Address, EhFrame, EndianVec, FrameTable, Writer};
 use object::write::{Object, SectionId, StandardSegment, Symbol, SymbolId, SymbolSection};
 use object::{Architecture, SectionFlags, SectionKind, SymbolFlags, SymbolKind, SymbolScope};
-use std::collections::HashMap;
 use std::ops::Range;
-use wasmtime_environ::obj::{self, LibCall};
-use wasmtime_environ::{Compiler, TripleExt, Unsigned};
+use wasmtime_environ::error::Result;
+use wasmtime_environ::{Compiler, TripleExt};
+use wasmtime_environ::{FuncKey, obj};
 
 const TEXT_SECTION_NAME: &[u8] = b".text";
 
@@ -63,13 +61,6 @@ pub struct ModuleTextBuilder<'a> {
     /// build to resolve relocations (calls) between functions.
     text: Box<dyn TextSectionBuilder>,
 
-    /// Symbols defined in the object for libcalls that relocations are applied
-    /// against.
-    ///
-    /// Note that this isn't typically used. It's only used for SSE-disabled
-    /// builds without SIMD on x86_64 right now.
-    libcall_symbols: HashMap<LibCall, SymbolId>,
-
     ctrl_plane: ControlPlane,
 }
 
@@ -111,7 +102,6 @@ impl<'a> ModuleTextBuilder<'a> {
             text_section,
             unwind_info: Default::default(),
             text,
-            libcall_symbols: HashMap::default(),
             ctrl_plane: ControlPlane::default(),
         }
     }
@@ -130,7 +120,7 @@ impl<'a> ModuleTextBuilder<'a> {
         &mut self,
         name: &str,
         compiled_func: &'a CompiledFunction,
-        resolve_reloc_target: impl Fn(wasmtime_environ::RelocationTarget) -> usize,
+        resolve_reloc_target: impl Fn(wasmtime_environ::FuncKey) -> usize,
     ) -> (SymbolId, Range<u64>) {
         let body = compiled_func.buffer.data();
         let alignment = compiled_func.alignment;
@@ -156,108 +146,49 @@ impl<'a> ModuleTextBuilder<'a> {
 
         for r in compiled_func.relocations() {
             let reloc_offset = off + u64::from(r.offset);
-            match r.reloc_target {
-                // Relocations against user-defined functions means that this is
-                // a relocation against a module-local function, typically a
-                // call between functions. The `text` field is given priority to
-                // resolve this relocation before we actually emit an object
-                // file, but if it can't handle it then we pass through the
-                // relocation.
-                RelocationTarget::Wasm(_) | RelocationTarget::Builtin(_) => {
-                    let target = resolve_reloc_target(r.reloc_target);
-                    if self
-                        .text
-                        .resolve_reloc(reloc_offset, r.reloc, r.addend, target)
-                    {
-                        continue;
-                    }
 
-                    // At this time it's expected that all relocations are
-                    // handled by `text.resolve_reloc`, and anything that isn't
-                    // handled is a bug in `text.resolve_reloc` or something
-                    // transitively there. If truly necessary, though, then this
-                    // loop could also be updated to forward the relocation to
-                    // the final object file as well.
-                    panic!(
-                        "unresolved relocation could not be processed against \
-                         {:?}: {r:?}",
-                        r.reloc_target,
-                    );
+            // This relocation is used to fill in which hostcall id is
+            // desired within the `call_indirect_host` opcode of Pulley
+            // itself. The relocation target is the start of the instruction
+            // and the goal is to insert the static signature number, `n`,
+            // into the instruction.
+            //
+            // At this time the instruction looks like:
+            //
+            //      +------+------+------+------+
+            //      | OP   | OP_EXTENDED |  N   |
+            //      +------+------+------+------+
+            //
+            // This 4-byte encoding has `OP` indicating this is an "extended
+            // opcode" where `OP_EXTENDED` is a 16-bit extended opcode.
+            // The `N` byte is the index of the signature being called and
+            // is what's b eing filled in.
+            //
+            // See the `test_call_indirect_host_width` in
+            // `pulley/tests/all.rs` for this guarantee as well.
+            if let FuncKey::PulleyHostCall(host_call) = r.reloc_target {
+                #[cfg(feature = "pulley")]
+                {
+                    use pulley_interpreter::encode::Encode;
+                    assert_eq!(pulley_interpreter::CallIndirectHost::WIDTH, 4);
                 }
+                let n = host_call.index();
+                let byte = u8::try_from(n).unwrap();
+                self.text.write(reloc_offset + 3, &[byte]);
+                continue;
+            }
 
-                // Relocations against libcalls are not common at this time and
-                // are only used in non-default configurations that disable wasm
-                // SIMD, disable SSE features, and for wasm modules that still
-                // use floating point operations.
-                //
-                // Currently these relocations are all expected to be absolute
-                // 8-byte relocations so that's asserted here and then encoded
-                // directly into the object as a normal object relocation. This
-                // is processed at module load time to resolve the relocations.
-                RelocationTarget::HostLibcall(call) => {
-                    let symbol = *self.libcall_symbols.entry(call).or_insert_with(|| {
-                        self.obj.add_symbol(Symbol {
-                            name: call.symbol().as_bytes().to_vec(),
-                            value: 0,
-                            size: 0,
-                            kind: SymbolKind::Text,
-                            scope: SymbolScope::Linkage,
-                            weak: false,
-                            section: SymbolSection::Undefined,
-                            flags: SymbolFlags::None,
-                        })
-                    });
-                    let flags = match r.reloc {
-                        Reloc::Abs8 => object::RelocationFlags::Generic {
-                            encoding: object::RelocationEncoding::Generic,
-                            kind: object::RelocationKind::Absolute,
-                            size: 64,
-                        },
-                        other => unimplemented!("unimplemented relocation kind {other:?}"),
-                    };
-                    self.obj
-                        .add_relocation(
-                            self.text_section,
-                            object::write::Relocation {
-                                symbol,
-                                flags,
-                                offset: reloc_offset,
-                                addend: r.addend,
-                            },
-                        )
-                        .unwrap();
-                }
+            let target = resolve_reloc_target(r.reloc_target);
+            if self
+                .text
+                .resolve_reloc(reloc_offset, r.reloc, r.addend, target)
+            {
+                continue;
+            }
 
-                // This relocation is used to fill in which hostcall id is
-                // desired within the `call_indirect_host` opcode of Pulley
-                // itself. The relocation target is the start of the instruction
-                // and the goal is to insert the static signature number, `n`,
-                // into the instruction.
-                //
-                // At this time the instruction looks like:
-                //
-                //      +------+------+------+------+
-                //      | OP   | OP_EXTENDED |  N   |
-                //      +------+------+------+------+
-                //
-                // This 4-byte encoding has `OP` indicating this is an "extended
-                // opcode" where `OP_EXTENDED` is a 16-bit extended opcode.
-                // The `N` byte is the index of the signature being called and
-                // is what's b eing filled in.
-                //
-                // See the `test_call_indirect_host_width` in
-                // `pulley/tests/all.rs` for this guarantee as well.
-                RelocationTarget::PulleyHostcall(n) => {
-                    #[cfg(feature = "pulley")]
-                    {
-                        use pulley_interpreter::encode::Encode;
-                        assert_eq!(pulley_interpreter::CallIndirectHost::WIDTH, 4);
-                    }
-                    let byte = u8::try_from(n).unwrap();
-                    self.text.write(reloc_offset + 3, &[byte]);
-                }
-            };
+            panic!("failed to resolve relocation: {r:?} -> {target}");
         }
+
         (symbol_id, off..off + body_len)
     }
 
@@ -287,9 +218,12 @@ impl<'a> ModuleTextBuilder<'a> {
     ///
     /// Note that this will also write out the unwind information sections if
     /// necessary.
-    pub fn finish(mut self) {
+    pub fn finish(mut self, postprocess_text: impl FnOnce(&mut [u8])) {
         // Finish up the text section now that we're done adding functions.
-        let text = self.text.finish(&mut self.ctrl_plane);
+        let mut text = self.text.finish(&mut self.ctrl_plane);
+
+        postprocess_text(&mut text[..]);
+
         self.obj
             .section_mut(self.text_section)
             .set_data(text, text_align(self.compiler));
@@ -599,7 +533,7 @@ impl<'a> UnwindInfoBuilder<'a> {
             // unwinders just use this constant for a relative addition with the
             // address of the FDE, which means that the sign doesn't actually
             // matter.
-            let fde = unwind_info.to_fde(Address::Constant(actual_offset.unsigned()));
+            let fde = unwind_info.to_fde(Address::Constant(actual_offset.cast_unsigned()));
             table.add_fde(cie_id, fde);
         }
         let endian = match compiler.triple().endianness().unwrap() {
