@@ -1,6 +1,6 @@
 use crate::component::{MAX_FLAT_PARAMS, MAX_FLAT_RESULTS};
-use crate::{prelude::*, TypeTrace};
 use crate::{EntityType, ModuleInternedTypeIndex, ModuleTypes, PrimaryMap};
+use crate::{TypeTrace, prelude::*};
 use core::hash::{Hash, Hasher};
 use core::ops::Index;
 use serde_derive::{Deserialize, Serialize};
@@ -91,13 +91,16 @@ indices! {
     pub struct TypeListIndex(u32);
     /// Index pointing to a future type in the component model.
     pub struct TypeFutureIndex(u32);
+
     /// Index pointing to a future table within a component.
     ///
     /// This is analogous to `TypeResourceTableIndex` in that it tracks
     /// ownership of futures within each (sub)component instance.
     pub struct TypeFutureTableIndex(u32);
+
     /// Index pointing to a stream type in the component model.
     pub struct TypeStreamIndex(u32);
+
     /// Index pointing to a stream table within a component.
     ///
     /// This is analogous to `TypeResourceTableIndex` in that it tracks
@@ -117,9 +120,6 @@ indices! {
     /// not just a subcomponent.
     pub struct TypeComponentGlobalErrorContextTableIndex(u32);
 
-    /// Index pointing to an interned `task.return` type within a component.
-    pub struct TypeTaskReturnIndex(u32);
-
     /// Index pointing to a resource table within a component.
     ///
     /// This is a Wasmtime-specific type index which isn't part of the component
@@ -222,6 +222,14 @@ indices! {
     /// Same as `RuntimeMemoryIndex` except for the `post-return` function.
     pub struct RuntimePostReturnIndex(u32);
 
+    /// Index representing a table extracted from a wasm instance which is
+    /// stored in a `VMComponentContext`. This is used to deduplicate references
+    /// to the same table when it's only stored once in a `VMComponentContext`.
+    ///
+    /// This does not correspond to anything in the binary format for the
+    /// component model.
+    pub struct RuntimeTableIndex(u32);
+
     /// Index for all trampolines that are compiled in Cranelift for a
     /// component.
     ///
@@ -232,6 +240,17 @@ indices! {
 
     /// An index into `Component::export_items` at the end of compilation.
     pub struct ExportIndex(u32);
+
+    /// An index into `Component::options` at the end of compilation.
+    pub struct OptionsIndex(u32);
+
+    /// An index that doesn't actually index into a list but instead represents
+    /// a unique counter.
+    ///
+    /// This is used for "abstract" resources which aren't actually instantiated
+    /// in the component model. For example this represents a resource in a
+    /// component or instance type, but not an actual concrete instance.
+    pub struct AbstractResourceIndex(u32);
 }
 
 // Reexport for convenience some core-wasm indices which are also used in the
@@ -241,7 +260,7 @@ pub use crate::{FuncIndex, GlobalIndex, MemoryIndex, TableIndex};
 /// Equivalent of `EntityIndex` but for the component model instead of core
 /// wasm.
 #[derive(Debug, Clone, Copy)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum ComponentItem {
     Func(ComponentFuncIndex),
     Module(ModuleIndex),
@@ -277,7 +296,6 @@ pub struct ComponentTypes {
     pub(super) stream_tables: PrimaryMap<TypeStreamTableIndex, TypeStreamTable>,
     pub(super) error_context_tables:
         PrimaryMap<TypeComponentLocalErrorContextTableIndex, TypeErrorContextTable>,
-    pub(super) task_returns: PrimaryMap<TypeTaskReturnIndex, TypeTaskReturn>,
 }
 
 impl TypeTrace for ComponentTypes {
@@ -532,26 +550,14 @@ pub struct TypeComponentInstance {
 /// A component function type in the component model.
 #[derive(Serialize, Deserialize, Clone, Hash, Eq, PartialEq, Debug)]
 pub struct TypeFunc {
+    /// Whether or not this is an async function.
+    pub async_: bool,
     /// Names of parameters.
     pub param_names: Vec<String>,
     /// Parameters to the function represented as a tuple.
     pub params: TypeTupleIndex,
     /// Results of the function represented as a tuple.
     pub results: TypeTupleIndex,
-    /// Expected core func type for memory32 `task.return` calls for this function.
-    pub task_return_type32: TypeTaskReturnIndex,
-    /// Expected core func type for memory64 `task.return` calls for this function.
-    pub task_return_type64: TypeTaskReturnIndex,
-}
-
-/// A core type representing the expected `task.return` signature for a
-/// component function.
-#[derive(Serialize, Deserialize, Clone, Hash, Eq, PartialEq, Debug)]
-pub struct TypeTaskReturn {
-    /// Core type parameters for the signature.
-    ///
-    /// Note that `task.return` never returns results.
-    pub params: Vec<FlatType>,
 }
 
 /// All possible interface types that values can have.
@@ -561,7 +567,7 @@ pub struct TypeTaskReturn {
 /// forms where for non-primitive types a `ComponentTypes` structure is used to
 /// lookup further information based on the index found here.
 #[derive(Serialize, Deserialize, Copy, Clone, Hash, Eq, PartialEq, Debug)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum InterfaceType {
     Bool,
     S8,
@@ -591,26 +597,6 @@ pub enum InterfaceType {
     ErrorContext(TypeComponentLocalErrorContextTableIndex),
 }
 
-impl From<&wasmparser::PrimitiveValType> for InterfaceType {
-    fn from(ty: &wasmparser::PrimitiveValType) -> InterfaceType {
-        match ty {
-            wasmparser::PrimitiveValType::Bool => InterfaceType::Bool,
-            wasmparser::PrimitiveValType::S8 => InterfaceType::S8,
-            wasmparser::PrimitiveValType::U8 => InterfaceType::U8,
-            wasmparser::PrimitiveValType::S16 => InterfaceType::S16,
-            wasmparser::PrimitiveValType::U16 => InterfaceType::U16,
-            wasmparser::PrimitiveValType::S32 => InterfaceType::S32,
-            wasmparser::PrimitiveValType::U32 => InterfaceType::U32,
-            wasmparser::PrimitiveValType::S64 => InterfaceType::S64,
-            wasmparser::PrimitiveValType::U64 => InterfaceType::U64,
-            wasmparser::PrimitiveValType::F32 => InterfaceType::Float32,
-            wasmparser::PrimitiveValType::F64 => InterfaceType::Float64,
-            wasmparser::PrimitiveValType::Char => InterfaceType::Char,
-            wasmparser::PrimitiveValType::String => InterfaceType::String,
-        }
-    }
-}
-
 /// Bye information about a type in the canonical ABI, with metadata for both
 /// memory32 and memory64-based types.
 #[derive(Serialize, Deserialize, Clone, Hash, Eq, PartialEq, Debug)]
@@ -650,16 +636,12 @@ const fn align_to(a: u32, b: u32) -> u32 {
 }
 
 const fn max(a: u32, b: u32) -> u32 {
-    if a > b {
-        a
-    } else {
-        b
-    }
+    if a > b { a } else { b }
 }
 
 impl CanonicalAbiInfo {
     /// ABI information for zero-sized types.
-    const ZERO: CanonicalAbiInfo = CanonicalAbiInfo {
+    pub const ZERO: CanonicalAbiInfo = CanonicalAbiInfo {
         size32: 0,
         align32: 1,
         size64: 0,
@@ -883,11 +865,7 @@ impl CanonicalAbiInfo {
     /// doesn't exceed the `max` specified.
     pub fn flat_count(&self, max: usize) -> Option<usize> {
         let flat = usize::from(self.flat_count?);
-        if flat > max {
-            None
-        } else {
-            Some(flat)
-        }
+        if flat > max { None } else { Some(flat) }
     }
 }
 
@@ -942,7 +920,7 @@ impl VariantInfo {
 
 mod serde_discrim_size {
     use super::DiscriminantSize;
-    use serde::{de::Error, Deserialize, Deserializer, Serialize, Serializer};
+    use serde::{Deserialize, Deserializer, Serialize, Serializer, de::Error};
 
     pub fn serialize<S>(disc: &DiscriminantSize, ser: S) -> Result<S::Ok, S::Error>
     where
@@ -1138,15 +1116,56 @@ pub struct TypeErrorContextTable {
 
 /// Metadata about a resource table added to a component.
 #[derive(Serialize, Deserialize, Clone, Hash, Eq, PartialEq, Debug)]
-pub struct TypeResourceTable {
-    /// The original resource that this table contains.
+pub enum TypeResourceTable {
+    /// This resource is for an actual concrete resource which has runtime state
+    /// associated with it.
     ///
-    /// This is used when destroying resources within this table since this
-    /// original definition will know how to execute destructors.
-    pub ty: ResourceIndex,
+    /// This is used for any resource which might actually enter a component.
+    /// For example when a resource is either imported or defined in a component
+    /// it'll get this case.
+    Concrete {
+        /// The original resource that this table contains.
+        ///
+        /// This is used when destroying resources within this table since this
+        /// original definition will know how to execute destructors.
+        ty: ResourceIndex,
 
-    /// The component instance that contains this resource table.
-    pub instance: RuntimeComponentInstanceIndex,
+        /// The component instance that contains this resource table.
+        instance: RuntimeComponentInstanceIndex,
+    },
+
+    /// This table does not actually exist at runtime but instead represents
+    /// type information for an uninstantiable resource. This tracks, for
+    /// example, resources in component and instance types.
+    Abstract(AbstractResourceIndex),
+}
+
+impl TypeResourceTable {
+    /// Asserts that this is `TypeResourceTable::Concrete` and returns the `ty`
+    /// field.
+    ///
+    /// # Panics
+    ///
+    /// Panics if this is `TypeResourceTable::Abstract`.
+    pub fn unwrap_concrete_ty(&self) -> ResourceIndex {
+        match self {
+            TypeResourceTable::Concrete { ty, .. } => *ty,
+            TypeResourceTable::Abstract(_) => panic!("not a concrete resource table"),
+        }
+    }
+
+    /// Asserts that this is `TypeResourceTable::Concrete` and returns the
+    /// `instance` field.
+    ///
+    /// # Panics
+    ///
+    /// Panics if this is `TypeResourceTable::Abstract`.
+    pub fn unwrap_concrete_instance(&self) -> RuntimeComponentInstanceIndex {
+        match self {
+            TypeResourceTable::Concrete { instance, .. } => *instance,
+            TypeResourceTable::Abstract(_) => panic!("not a concrete resource table"),
+        }
+    }
 }
 
 /// Shape of a "list" interface type.
@@ -1172,11 +1191,7 @@ const fn add_flat(a: Option<u8>, b: Option<u8>) -> Option<u8> {
         },
         _ => return None,
     };
-    if sum > MAX {
-        None
-    } else {
-        Some(sum)
-    }
+    if sum > MAX { None } else { Some(sum) }
 }
 
 const fn max_flat(a: Option<u8>, b: Option<u8>) -> Option<u8> {
@@ -1214,7 +1229,7 @@ impl FlatTypes<'_> {
 // regardless to changes in the core wasm type system since this will only
 // ever use integers/floats for the foreseeable future.
 #[derive(Serialize, Deserialize, Hash, Debug, PartialEq, Eq, Copy, Clone)]
-#[allow(missing_docs, reason = "self-describing variants")]
+#[expect(missing_docs, reason = "self-describing variants")]
 pub enum FlatType {
     I32,
     I64,
