@@ -18,31 +18,34 @@ use serde_derive::{Deserialize, Serialize};
 #[cfg_attr(test, derive(arbitrary::Arbitrary))]
 pub enum FuncKeyKind {
     /// A Wasm-defined function.
-    DefinedWasmFunction = FuncKey::new_kind(0b000),
+    DefinedWasmFunction = FuncKey::new_kind(0b0000),
 
     /// A trampoline from an array-caller to the given Wasm-callee.
-    ArrayToWasmTrampoline = FuncKey::new_kind(0b001),
+    ArrayToWasmTrampoline = FuncKey::new_kind(0b0001),
 
     /// A trampoline from a Wasm-caller to an array-callee of the given type.
-    WasmToArrayTrampoline = FuncKey::new_kind(0b010),
+    WasmToArrayTrampoline = FuncKey::new_kind(0b0010),
 
     /// A trampoline from a Wasm-caller to the given builtin.
-    WasmToBuiltinTrampoline = FuncKey::new_kind(0b011),
+    WasmToBuiltinTrampoline = FuncKey::new_kind(0b0011),
+
+    /// A trampoline from the patchable ABI to the given builtin.
+    PatchableToBuiltinTrampoline = FuncKey::new_kind(0b0100),
 
     /// A Pulley-specific host call.
-    PulleyHostCall = FuncKey::new_kind(0b100),
+    PulleyHostCall = FuncKey::new_kind(0b0101),
 
     /// A Wasm-caller to component builtin trampoline.
     #[cfg(feature = "component-model")]
-    ComponentTrampoline = FuncKey::new_kind(0b101),
+    ComponentTrampoline = FuncKey::new_kind(0b0110),
 
     /// A Wasm-caller to array-callee `resource.drop` trampoline.
     #[cfg(feature = "component-model")]
-    ResourceDropTrampoline = FuncKey::new_kind(0b110),
+    ResourceDropTrampoline = FuncKey::new_kind(0b0111),
 
     /// A Wasmtime unsafe intrinsic function.
     #[cfg(feature = "component-model")]
-    UnsafeIntrinsic = FuncKey::new_kind(0b111),
+    UnsafeIntrinsic = FuncKey::new_kind(0b1000),
 }
 
 impl From<FuncKeyKind> for u32 {
@@ -66,6 +69,9 @@ impl FuncKeyKind {
             x if x == Self::ArrayToWasmTrampoline.into() => Self::ArrayToWasmTrampoline,
             x if x == Self::WasmToArrayTrampoline.into() => Self::WasmToArrayTrampoline,
             x if x == Self::WasmToBuiltinTrampoline.into() => Self::WasmToBuiltinTrampoline,
+            x if x == Self::PatchableToBuiltinTrampoline.into() => {
+                Self::PatchableToBuiltinTrampoline
+            }
             x if x == Self::PulleyHostCall.into() => Self::PulleyHostCall,
 
             #[cfg(feature = "component-model")]
@@ -122,6 +128,7 @@ impl FuncKeyNamespace {
             FuncKeyKind::DefinedWasmFunction | FuncKeyKind::ArrayToWasmTrampoline => Self(raw),
             FuncKeyKind::WasmToArrayTrampoline
             | FuncKeyKind::WasmToBuiltinTrampoline
+            | FuncKeyKind::PatchableToBuiltinTrampoline
             | FuncKeyKind::PulleyHostCall => {
                 assert_eq!(raw & FuncKey::MODULE_MASK, 0);
                 Self(raw)
@@ -192,6 +199,10 @@ pub enum Abi {
     Wasm = 0,
     /// The "array" ABI, or suitable to be an `array_call` field.
     Array = 1,
+    /// The "patchable" ABI. Signature same as Wasm ABI, but
+    /// Cranelift-level (machine-level) ABI is different (no
+    /// clobbers).
+    Patchable = 2,
 }
 
 #[cfg(feature = "component-model")]
@@ -200,6 +211,7 @@ impl Abi {
         match raw {
             x if x == Self::Wasm.into_raw() => Self::Wasm,
             x if x == Self::Array.into_raw() => Self::Array,
+            x if x == Self::Patchable.into_raw() => Self::Patchable,
             _ => panic!("invalid raw representation passed to `Abi::from_raw`: {raw}"),
         }
     }
@@ -228,6 +240,9 @@ pub enum FuncKey {
     /// A Pulley-specific host call.
     PulleyHostCall(HostCall),
 
+    /// A trampoline from the patchable ABI to the given builtin.
+    PatchableToBuiltinTrampoline(BuiltinFunctionIndex),
+
     /// A Wasm-caller to component builtin trampoline.
     #[cfg(feature = "component-model")]
     ComponentTrampoline(Abi, component::TrampolineIndex),
@@ -258,7 +273,7 @@ impl PartialOrd for FuncKey {
 }
 
 impl FuncKey {
-    const KIND_BITS: u32 = 3;
+    const KIND_BITS: u32 = 4;
     const KIND_OFFSET: u32 = 32 - Self::KIND_BITS;
     const KIND_MASK: u32 = ((1 << Self::KIND_BITS) - 1) << Self::KIND_OFFSET;
     const MODULE_MASK: u32 = !Self::KIND_MASK;
@@ -294,6 +309,11 @@ impl FuncKey {
                 let index = builtin.index();
                 (namespace, index)
             }
+            FuncKey::PatchableToBuiltinTrampoline(builtin) => {
+                let namespace = FuncKeyKind::PatchableToBuiltinTrampoline.into_raw();
+                let index = builtin.index();
+                (namespace, index)
+            }
             FuncKey::PulleyHostCall(host_call) => {
                 let namespace = FuncKeyKind::PulleyHostCall.into_raw();
                 let index = host_call.index();
@@ -341,13 +361,14 @@ impl FuncKey {
         self.into_parts().1
     }
 
-    /// Get ABI of the function that this key is definining.
+    /// Get ABI of the function that this key is defining.
     pub fn abi(self) -> Abi {
         match self {
             FuncKey::DefinedWasmFunction(_, _) => Abi::Wasm,
             FuncKey::ArrayToWasmTrampoline(_, _) => Abi::Array,
             FuncKey::WasmToArrayTrampoline(_) => Abi::Wasm,
             FuncKey::WasmToBuiltinTrampoline(_) => Abi::Wasm,
+            FuncKey::PatchableToBuiltinTrampoline(_) => Abi::Patchable,
             FuncKey::PulleyHostCall(_) => Abi::Wasm,
             #[cfg(feature = "component-model")]
             FuncKey::ComponentTrampoline(abi, _) => abi,
@@ -412,6 +433,11 @@ impl FuncKey {
                 let builtin = BuiltinFunctionIndex::from_u32(b);
                 Self::WasmToBuiltinTrampoline(builtin)
             }
+            FuncKeyKind::PatchableToBuiltinTrampoline => {
+                assert_eq!(a & Self::MODULE_MASK, 0);
+                let builtin = BuiltinFunctionIndex::from_u32(b);
+                Self::PatchableToBuiltinTrampoline(builtin)
+            }
             FuncKeyKind::PulleyHostCall => {
                 assert_eq!(a & Self::MODULE_MASK, 0);
                 let host_call = HostCall::from_index(b);
@@ -533,6 +559,7 @@ impl FuncKey {
             Self::DefinedWasmFunction(..) | Self::ArrayToWasmTrampoline(..) => false,
             Self::WasmToArrayTrampoline(..)
             | Self::WasmToBuiltinTrampoline(..)
+            | Self::PatchableToBuiltinTrampoline(..)
             | Self::PulleyHostCall(..) => true,
             #[cfg(feature = "component-model")]
             Self::ComponentTrampoline(..)
