@@ -10,10 +10,7 @@
 //      _padding: u32, // (On 64-bit systems)
 //      vm_store_context: *const VMStoreContext,
 //      builtin_functions: *mut VMBuiltinFunctionsArray,
-//      callee: *mut VMFunctionBody,
 //      epoch_ptr: *mut AtomicU64,
-//      gc_heap_base: *mut u8,
-//      gc_heap_bound: *mut u8,
 //      gc_heap_data: *mut T, // Collector-specific pointer
 //      type_ids: *const VMSharedTypeIndex,
 //
@@ -27,7 +24,7 @@
 //      memories: [*mut VMMemoryDefinition; module.num_defined_memories],
 //      owned_memories: [VMMemoryDefinition; module.num_owned_memories],
 //      imported_functions: [VMFunctionImport; module.num_imported_functions],
-//      imported_tables: [VMTable; module.num_imported_tables],
+//      imported_tables: [VMTableImport; module.num_imported_tables],
 //      imported_globals: [VMGlobalImport; module.num_imported_globals],
 //      imported_tags: [VMTagImport; module.num_imported_tags],
 //      tables: [VMTableDefinition; module.num_defined_tables],
@@ -164,6 +161,12 @@ pub trait PtrSize {
         4
     }
 
+    /// This is the size of the largest value type (i.e. a V128).
+    #[inline]
+    fn maximum_value_size(&self) -> u8 {
+        self.size_of_vmglobal_definition()
+    }
+
     // Offsets within `VMStoreContext`
 
     /// Return the offset of the `fuel_consumed` field of `VMStoreContext`
@@ -184,19 +187,60 @@ pub trait PtrSize {
         self.vmstore_context_epoch_deadline() + 8
     }
 
-    /// Return the offset of the `last_wasm_exit_fp` field of `VMStoreContext`.
-    fn vmstore_context_last_wasm_exit_fp(&self) -> u8 {
+    /// Return the offset of the `gc_heap` field of `VMStoreContext`.
+    #[inline]
+    fn vmstore_context_gc_heap(&self) -> u8 {
         self.vmstore_context_stack_limit() + self.size()
     }
 
+    /// Return the offset of the `gc_heap.base` field within a `VMStoreContext`.
+    fn vmstore_context_gc_heap_base(&self) -> u8 {
+        let offset = self.vmstore_context_gc_heap() + self.vmmemory_definition_base();
+        debug_assert!(offset < self.vmstore_context_last_wasm_exit_trampoline_fp());
+        offset
+    }
+
+    /// Return the offset of the `gc_heap.current_length` field within a `VMStoreContext`.
+    fn vmstore_context_gc_heap_current_length(&self) -> u8 {
+        let offset = self.vmstore_context_gc_heap() + self.vmmemory_definition_current_length();
+        debug_assert!(offset < self.vmstore_context_last_wasm_exit_trampoline_fp());
+        offset
+    }
+
+    /// Return the offset of the `last_wasm_exit_trampoline_fp` field
+    /// of `VMStoreContext`.
+    fn vmstore_context_last_wasm_exit_trampoline_fp(&self) -> u8 {
+        self.vmstore_context_gc_heap() + self.size_of_vmmemory_definition()
+    }
+
     /// Return the offset of the `last_wasm_exit_pc` field of `VMStoreContext`.
     fn vmstore_context_last_wasm_exit_pc(&self) -> u8 {
-        self.vmstore_context_last_wasm_exit_fp() + self.size()
+        self.vmstore_context_last_wasm_exit_trampoline_fp() + self.size()
+    }
+
+    /// Return the offset of the `last_wasm_entry_sp` field of `VMStoreContext`.
+    fn vmstore_context_last_wasm_entry_sp(&self) -> u8 {
+        self.vmstore_context_last_wasm_exit_pc() + self.size()
     }
 
     /// Return the offset of the `last_wasm_entry_fp` field of `VMStoreContext`.
     fn vmstore_context_last_wasm_entry_fp(&self) -> u8 {
-        self.vmstore_context_last_wasm_exit_pc() + self.size()
+        self.vmstore_context_last_wasm_entry_sp() + self.size()
+    }
+
+    /// Return the offset of the `last_wasm_entry_trap_handler` field of `VMStoreContext`.
+    fn vmstore_context_last_wasm_entry_trap_handler(&self) -> u8 {
+        self.vmstore_context_last_wasm_entry_fp() + self.size()
+    }
+
+    /// Return the offset of the `stack_chain` field of `VMStoreContext`.
+    fn vmstore_context_stack_chain(&self) -> u8 {
+        self.vmstore_context_last_wasm_entry_trap_handler() + self.size()
+    }
+
+    /// Return the offset of the `stack_chain` field of `VMStoreContext`.
+    fn vmstore_context_store_data(&self) -> u8 {
+        self.vmstore_context_stack_chain() + self.size_of_vmstack_chain()
     }
 
     // Offsets within `VMMemoryDefinition`
@@ -236,6 +280,144 @@ pub trait PtrSize {
         .unwrap()
     }
 
+    /// Return the size of `VMStackChain`.
+    fn size_of_vmstack_chain(&self) -> u8 {
+        2 * self.size()
+    }
+
+    // Offsets within `VMStackLimits`
+
+    /// Return the offset of `VMStackLimits::stack_limit`.
+    fn vmstack_limits_stack_limit(&self) -> u8 {
+        0
+    }
+
+    /// Return the offset of `VMStackLimits::last_wasm_entry_fp`.
+    fn vmstack_limits_last_wasm_entry_fp(&self) -> u8 {
+        self.size()
+    }
+
+    // Offsets within `VMHostArray`
+
+    /// Return the offset of `VMHostArray::length`.
+    fn vmhostarray_length(&self) -> u8 {
+        0
+    }
+
+    /// Return the offset of `VMHostArray::capacity`.
+    fn vmhostarray_capacity(&self) -> u8 {
+        4
+    }
+
+    /// Return the offset of `VMHostArray::data`.
+    fn vmhostarray_data(&self) -> u8 {
+        8
+    }
+
+    /// Return the size of `VMHostArray`.
+    fn size_of_vmhostarray(&self) -> u8 {
+        8 + self.size()
+    }
+
+    // Offsets within `VMCommonStackInformation`
+
+    /// Return the offset of `VMCommonStackInformation::limits`.
+    fn vmcommon_stack_information_limits(&self) -> u8 {
+        0 * self.size()
+    }
+
+    /// Return the offset of `VMCommonStackInformation::state`.
+    fn vmcommon_stack_information_state(&self) -> u8 {
+        2 * self.size()
+    }
+
+    /// Return the offset of `VMCommonStackInformation::handlers`.
+    fn vmcommon_stack_information_handlers(&self) -> u8 {
+        u8::try_from(align(
+            self.vmcommon_stack_information_state() as u32 + 4,
+            u32::from(self.size()),
+        ))
+        .unwrap()
+    }
+
+    /// Return the offset of `VMCommonStackInformation::first_switch_handler_index`.
+    fn vmcommon_stack_information_first_switch_handler_index(&self) -> u8 {
+        self.vmcommon_stack_information_handlers() + self.size_of_vmhostarray()
+    }
+
+    /// Return the size of `VMCommonStackInformation`.
+    fn size_of_vmcommon_stack_information(&self) -> u8 {
+        u8::try_from(align(
+            self.vmcommon_stack_information_first_switch_handler_index() as u32 + 4,
+            u32::from(self.size()),
+        ))
+        .unwrap()
+    }
+
+    // Offsets within `VMContObj`
+
+    /// Return the offset of `VMContObj::contref`
+    fn vmcontobj_contref(&self) -> u8 {
+        0
+    }
+
+    /// Return the offset of `VMContObj::revision`
+    fn vmcontobj_revision(&self) -> u8 {
+        self.size()
+    }
+
+    /// Return the size of `VMContObj`.
+    fn size_of_vmcontobj(&self) -> u8 {
+        u8::try_from(align(
+            u32::from(self.vmcontobj_revision())
+                + u32::try_from(core::mem::size_of::<usize>()).unwrap(),
+            u32::from(self.size()),
+        ))
+        .unwrap()
+    }
+
+    // Offsets within `VMContRef`
+
+    /// Return the offset of `VMContRef::common_stack_information`.
+    fn vmcontref_common_stack_information(&self) -> u8 {
+        0 * self.size()
+    }
+
+    /// Return the offset of `VMContRef::parent_chain`.
+    fn vmcontref_parent_chain(&self) -> u8 {
+        u8::try_from(align(
+            (self.vmcontref_common_stack_information() + self.size_of_vmcommon_stack_information())
+                as u32,
+            u32::from(self.size()),
+        ))
+        .unwrap()
+    }
+
+    /// Return the offset of `VMContRef::last_ancestor`.
+    fn vmcontref_last_ancestor(&self) -> u8 {
+        self.vmcontref_parent_chain() + 2 * self.size()
+    }
+
+    /// Return the offset of `VMContRef::revision`.
+    fn vmcontref_revision(&self) -> u8 {
+        self.vmcontref_last_ancestor() + self.size()
+    }
+
+    /// Return the offset of `VMContRef::stack`.
+    fn vmcontref_stack(&self) -> u8 {
+        self.vmcontref_revision() + self.size()
+    }
+
+    /// Return the offset of `VMContRef::args`.
+    fn vmcontref_args(&self) -> u8 {
+        self.vmcontref_stack() + 3 * self.size()
+    }
+
+    /// Return the offset of `VMContRef::values`.
+    fn vmcontref_values(&self) -> u8 {
+        self.vmcontref_args() + self.size_of_vmhostarray()
+    }
+
     /// Return the offset to the `magic` value in this `VMContext`.
     #[inline]
     fn vmctx_magic(&self) -> u8 {
@@ -257,29 +439,11 @@ pub trait PtrSize {
         self.vmctx_store_context() + self.size()
     }
 
-    /// Return the offset to the `callee` member in this `VMContext`.
-    #[inline]
-    fn vmctx_callee(&self) -> u8 {
-        self.vmctx_builtin_functions() + self.size()
-    }
-
     /// Return the offset to the `*const AtomicU64` epoch-counter
     /// pointer.
     #[inline]
     fn vmctx_epoch_ptr(&self) -> u8 {
-        self.vmctx_callee() + self.size()
-    }
-
-    /// Return the offset to the GC heap base in this `VMContext`.
-    #[inline]
-    fn vmctx_gc_heap_base(&self) -> u8 {
-        self.vmctx_epoch_ptr() + self.size()
-    }
-
-    /// Return the offset to the GC heap bound in this `VMContext`.
-    #[inline]
-    fn vmctx_gc_heap_bound(&self) -> u8 {
-        self.vmctx_gc_heap_base() + self.size()
+        self.vmctx_builtin_functions() + self.size()
     }
 
     /// Return the offset to the `*mut T` collector-specific data.
@@ -288,7 +452,7 @@ pub trait PtrSize {
     /// fit.
     #[inline]
     fn vmctx_gc_heap_data(&self) -> u8 {
-        self.vmctx_gc_heap_bound() + self.size()
+        self.vmctx_epoch_ptr() + self.size()
     }
 
     /// The offset of the `type_ids` array pointer.
@@ -520,7 +684,7 @@ impl<P: PtrSize> From<VMOffsetsFields<P>> for VMOffsets<P> {
             size(imported_functions)
                 = cmul(ret.num_imported_functions, ret.size_of_vmfunction_import()),
             size(imported_tables)
-                = cmul(ret.num_imported_tables, ret.size_of_vmtable()),
+                = cmul(ret.num_imported_tables, ret.size_of_vmtable_import()),
             size(imported_globals)
                 = cmul(ret.num_imported_globals, ret.size_of_vmglobal_import()),
             size(imported_tags)
@@ -578,25 +742,31 @@ impl<P: PtrSize> VMOffsets<P> {
     }
 }
 
-/// Offsets for `VMTable`.
+/// Offsets for `VMTableImport`.
 impl<P: PtrSize> VMOffsets<P> {
     /// The offset of the `from` field.
     #[inline]
-    pub fn vmtable_from(&self) -> u8 {
+    pub fn vmtable_import_from(&self) -> u8 {
         0 * self.pointer_size()
     }
 
     /// The offset of the `vmctx` field.
     #[inline]
-    pub fn vmtable_vmctx(&self) -> u8 {
+    pub fn vmtable_import_vmctx(&self) -> u8 {
         1 * self.pointer_size()
     }
 
-    /// Return the size of `VMTable`.
+    /// The offset of the `index` field.
     #[inline]
-    pub fn size_of_vmtable(&self) -> u8 {
+    pub fn vmtable_import_index(&self) -> u8 {
         2 * self.pointer_size()
     }
+
+    /// Return the size of `VMTableImport`.
+    #[inline]
+    pub fn size_of_vmtable_import(&self) -> u8 {
+        3 * self.pointer_size()
+    }
 }
 
 /// Offsets for `VMTableDefinition`.
@@ -639,6 +809,12 @@ impl<P: PtrSize> VMOffsets<P> {
         1 * self.pointer_size()
     }
 
+    /// The offset of the `index` field.
+    #[inline]
+    pub fn vmmemory_import_index(&self) -> u8 {
+        2 * self.pointer_size()
+    }
+
     /// Return the size of `VMMemoryImport`.
     #[inline]
     pub fn size_of_vmmemory_import(&self) -> u8 {
@@ -657,7 +833,8 @@ impl<P: PtrSize> VMOffsets<P> {
     /// Return the size of `VMGlobalImport`.
     #[inline]
     pub fn size_of_vmglobal_import(&self) -> u8 {
-        1 * self.pointer_size()
+        // `VMGlobalImport` has two pointers plus 8 bytes for `VMGlobalKind`
+        2 * self.pointer_size() + 8
     }
 }
 
@@ -678,10 +855,22 @@ impl<P: PtrSize> VMOffsets<P> {
         0 * self.pointer_size()
     }
 
+    /// The offset of the `vmctx` field.
+    #[inline]
+    pub fn vmtag_import_vmctx(&self) -> u8 {
+        1 * self.pointer_size()
+    }
+
+    /// The offset of the `index` field.
+    #[inline]
+    pub fn vmtag_import_index(&self) -> u8 {
+        2 * self.pointer_size()
+    }
+
     /// Return the size of `VMTagImport`.
     #[inline]
     pub fn size_of_vmtag_import(&self) -> u8 {
-        1 * self.pointer_size()
+        3 * self.pointer_size()
     }
 }
 
@@ -771,7 +960,8 @@ impl<P: PtrSize> VMOffsets<P> {
     #[inline]
     pub fn vmctx_vmtable_import(&self, index: TableIndex) -> u32 {
         assert!(index.as_u32() < self.num_imported_tables);
-        self.vmctx_imported_tables_begin() + index.as_u32() * u32::from(self.size_of_vmtable())
+        self.vmctx_imported_tables_begin()
+            + index.as_u32() * u32::from(self.size_of_vmtable_import())
     }
 
     /// Return the offset to `VMMemoryImport` index `index`.
@@ -866,7 +1056,7 @@ impl<P: PtrSize> VMOffsets<P> {
     /// `index`.
     #[inline]
     pub fn vmctx_vmtable_from(&self, index: TableIndex) -> u32 {
-        self.vmctx_vmtable_import(index) + u32::from(self.vmtable_from())
+        self.vmctx_vmtable_import(index) + u32::from(self.vmtable_import_from())
     }
 
     /// Return the offset to the `base` field in `VMTableDefinition` index `index`.
@@ -887,12 +1077,6 @@ impl<P: PtrSize> VMOffsets<P> {
         self.vmctx_vmmemory_import(index) + u32::from(self.vmmemory_import_from())
     }
 
-    /// Return the offset to the `vmctx` field in `VMMemoryImport` index `index`.
-    #[inline]
-    pub fn vmctx_vmmemory_import_vmctx(&self, index: MemoryIndex) -> u32 {
-        self.vmctx_vmmemory_import(index) + u32::from(self.vmmemory_import_vmctx())
-    }
-
     /// Return the offset to the `base` field in `VMMemoryDefinition` index `index`.
     #[inline]
     pub fn vmctx_vmmemory_definition_base(&self, index: OwnedMemoryIndex) -> u32 {
@@ -917,6 +1101,40 @@ impl<P: PtrSize> VMOffsets<P> {
     pub fn vmctx_vmtag_import_from(&self, index: TagIndex) -> u32 {
         self.vmctx_vmtag_import(index) + u32::from(self.vmtag_import_from())
     }
+
+    /// Return the offset to the `vmctx` field in `VMTagImport` index `index`.
+    #[inline]
+    pub fn vmctx_vmtag_import_vmctx(&self, index: TagIndex) -> u32 {
+        self.vmctx_vmtag_import(index) + u32::from(self.vmtag_import_vmctx())
+    }
+
+    /// Return the offset to the `index` field in `VMTagImport` index `index`.
+    #[inline]
+    pub fn vmctx_vmtag_import_index(&self, index: TagIndex) -> u32 {
+        self.vmctx_vmtag_import(index) + u32::from(self.vmtag_import_index())
+    }
+}
+
+/// Offsets for `VMGcHeader`.
+impl<P: PtrSize> VMOffsets<P> {
+    /// Return the offset for the `VMGcHeader::kind` field.
+    #[inline]
+    pub fn vm_gc_header_kind(&self) -> u32 {
+        0
+    }
+
+    /// Return the offset for the `VMGcHeader`'s reserved bits.
+    #[inline]
+    pub fn vm_gc_header_reserved_bits(&self) -> u32 {
+        // NB: The reserved bits are the unused `VMGcKind` bits.
+        self.vm_gc_header_kind()
+    }
+
+    /// Return the offset for the `VMGcHeader::ty` field.
+    #[inline]
+    pub fn vm_gc_header_ty(&self) -> u32 {
+        self.vm_gc_header_kind() + 4
+    }
 }
 
 /// Offsets for `VMDrcHeader`.
@@ -928,22 +1146,11 @@ impl<P: PtrSize> VMOffsets<P> {
     pub fn vm_drc_header_ref_count(&self) -> u32 {
         8
     }
-}
 
-/// Offsets for `VMGcRefActivationsTable`.
-///
-/// These should only be used when the DRC collector is enabled.
-impl<P: PtrSize> VMOffsets<P> {
-    /// Return the offset for `VMGcRefActivationsTable::next`.
+    /// Return the offset for `VMDrcHeader::next_over_approximated_stack_root`.
     #[inline]
-    pub fn vm_gc_ref_activation_table_next(&self) -> u32 {
-        0
-    }
-
-    /// Return the offset for `VMGcRefActivationsTable::end`.
-    #[inline]
-    pub fn vm_gc_ref_activation_table_end(&self) -> u32 {
-        self.pointer_size().into()
+    pub fn vm_drc_header_next_over_approximated_stack_root(&self) -> u32 {
+        self.vm_drc_header_ref_count() + 8
     }
 }
 
