@@ -20,8 +20,11 @@ impl Run for Component {
 
 impl Backpressure for Component {
     fn set_backpressure(enabled: bool) {
-        #[expect(deprecated, reason = "will replace with backpressure.inc/dec soon")]
-        wit_bindgen::backpressure_set(enabled);
+        if enabled {
+            wit_bindgen::backpressure_inc();
+        } else {
+            wit_bindgen::backpressure_dec();
+        }
     }
     fn inc_backpressure() {
         wit_bindgen::backpressure_inc();
