@@ -1045,6 +1045,14 @@ impl Dir {
             .await?;
 
         match opened {
+            // Paper over a divergence between Windows and POSIX, where
+            // POSIX returns EISDIR if you open a directory with the
+            // WRITE flag: https://pubs.opengroup.org/onlinepubs/9699919799/functions/open.html#:~:text=EISDIR
+            #[cfg(windows)]
+            OpenResult::Dir(_) if flags.contains(DescriptorFlags::WRITE) => {
+                Err(ErrorCode::IsDirectory)
+            }
+
             OpenResult::Dir(dir) => Ok(Descriptor::Dir(Dir::new(
                 dir,
                 self.perms,
