@@ -32,11 +32,10 @@ use call_graph::CallGraph;
 #[cfg(feature = "component-model")]
 use wasmtime_environ::component::Translator;
 use wasmtime_environ::{
-    Abi, BuiltinFunctionIndex, CompiledFunctionBody, CompiledFunctionsTable,
-    CompiledFunctionsTableBuilder, CompiledModuleInfo, Compiler, DefinedFuncIndex, FilePos,
-    FinishedObject, FuncKey, FunctionBodyData, InliningCompiler, IntraModuleInlining,
-    ModuleEnvironment, ModuleTranslation, ModuleTypes, ModuleTypesBuilder, ObjectKind, PrimaryMap,
-    StaticModuleIndex, Tunables,
+    Abi, CompiledFunctionBody, CompiledFunctionsTable, CompiledFunctionsTableBuilder,
+    CompiledModuleInfo, Compiler, DefinedFuncIndex, FilePos, FinishedObject, FuncKey,
+    FunctionBodyData, InliningCompiler, IntraModuleInlining, ModuleEnvironment, ModuleTranslation,
+    ModuleTypes, ModuleTypesBuilder, ObjectKind, PrimaryMap, StaticModuleIndex, Tunables,
 };
 
 mod call_graph;
@@ -306,6 +305,7 @@ impl<'a> CompileInputs<'a> {
                         match abi {
                             Abi::Wasm => "wasm-call",
                             Abi::Array => "array-call",
+                            Abi::Patchable => "patchable-call",
                         },
                         intrinsic.name(),
                     );
@@ -334,6 +334,7 @@ impl<'a> CompileInputs<'a> {
                         match abi {
                             Abi::Wasm => "wasm-call",
                             Abi::Array => "array-call",
+                            Abi::Patchable => "patchable-call",
                         },
                         trampoline.symbol_name(),
                     );
@@ -626,7 +627,8 @@ the use case.
                 // in their own call frames.
                 FuncKey::ArrayToWasmTrampoline(..)
                 | FuncKey::WasmToArrayTrampoline(..)
-                | FuncKey::WasmToBuiltinTrampoline(..) => false,
+                | FuncKey::WasmToBuiltinTrampoline(..)
+                | FuncKey::PatchableToBuiltinTrampoline(..) => false,
                 #[cfg(feature = "component-model")]
                 FuncKey::ComponentTrampoline(..) | FuncKey::ResourceDropTrampoline => false,
 
@@ -917,10 +919,17 @@ fn compile_required_builtins(engine: &Engine, raw_outputs: &mut Vec<CompileOutpu
     let mut builtins = HashSet::new();
     let mut new_inputs: Vec<CompileInput<'_>> = Vec::new();
 
-    let compile_builtin = |builtin: BuiltinFunctionIndex| {
+    let compile_builtin = |key: FuncKey| {
         Box::new(move |compiler: &dyn Compiler| {
-            let key = FuncKey::WasmToBuiltinTrampoline(builtin);
-            let symbol = format!("wasmtime_builtin_{}", builtin.name());
+            let symbol = match key {
+                FuncKey::WasmToBuiltinTrampoline(builtin) => {
+                    format!("wasmtime_builtin_{}", builtin.name())
+                }
+                FuncKey::PatchableToBuiltinTrampoline(builtin) => {
+                    format!("wasmtime_patchable_builtin_{}", builtin.name())
+                }
+                _ => unreachable!(),
+            };
             let mut function = compiler
                 .compile_wasm_to_builtin(key, &symbol)
                 .with_context(|| format!("failed to compile `{symbol}`"))?;
@@ -940,10 +949,14 @@ fn compile_required_builtins(engine: &Engine, raw_outputs: &mut Vec<CompileOutpu
 
     for output in raw_outputs.iter() {
         for reloc in compiler.compiled_function_relocation_targets(&*output.function.code) {
-            if let FuncKey::WasmToBuiltinTrampoline(builtin) = reloc {
-                if builtins.insert(builtin) {
-                    new_inputs.push(compile_builtin(builtin));
+            match reloc {
+                FuncKey::WasmToBuiltinTrampoline(builtin)
+                | FuncKey::PatchableToBuiltinTrampoline(builtin) => {
+                    if builtins.insert(builtin) {
+                        new_inputs.push(compile_builtin(reloc));
+                    }
                 }
+                _ => {}
             }
         }
     }
