@@ -808,10 +808,11 @@ pub struct HashedEngineCompileEnv<'a>(pub &'a Engine);
 impl std::hash::Hash for HashedEngineCompileEnv<'_> {
     fn hash<H: std::hash::Hasher>(&self, hasher: &mut H) {
         // Hash the compiler's state based on its target and configuration.
-        let compiler = self.0.compiler();
-        compiler.triple().hash(hasher);
-        compiler.flags().hash(hasher);
-        compiler.isa_flags().hash(hasher);
+        if let Some(compiler) = self.0.compiler() {
+            compiler.triple().hash(hasher);
+            compiler.flags().hash(hasher);
+            compiler.isa_flags().hash(hasher);
+        }
 
         // Hash configuration state read for compilation
         let config = self.0.config();
