@@ -21,8 +21,6 @@
 //! exit FP and stopping once we reach the entry SP (meaning that the next older
 //! frame is a host frame).
 
-#[cfg(feature = "debug")]
-use crate::StoreContextMut;
 use crate::prelude::*;
 use crate::runtime::store::StoreOpaque;
 use crate::runtime::vm::stack_switching::VMStackChain;
@@ -32,6 +30,8 @@ use crate::runtime::vm::{
 };
 #[cfg(all(feature = "gc", feature = "stack-switching"))]
 use crate::vm::stack_switching::{VMContRef, VMStackState};
+#[cfg(feature = "debug")]
+use crate::{StoreContext, StoreContextMut};
 use core::ops::ControlFlow;
 use wasmtime_unwinder::Frame;
 
@@ -39,6 +39,14 @@ use wasmtime_unwinder::Frame;
 #[derive(Debug)]
 pub struct Backtrace(Vec<Frame>);
 
+/// One activation: information sufficient to trace an activation on a
+/// frame as long as that frame remains alive.
+pub(crate) struct Activation {
+    exit_pc: usize,
+    exit_fp: usize,
+    entry_trampoline_fp: usize,
+}
+
 impl Backtrace {
     /// Returns an empty backtrace
     pub fn empty() -> Backtrace {
@@ -69,23 +77,41 @@ impl Backtrace {
         trap_pc_and_fp: Option<(usize, usize)>,
     ) -> Backtrace {
         let mut frames = vec![];
+        let f = |activation: Activation| unsafe {
+            wasmtime_unwinder::visit_frames(
+                unwind,
+                activation.exit_pc,
+                activation.exit_fp,
+                activation.entry_trampoline_fp,
+                |frame| {
+                    frames.push(frame);
+                    ControlFlow::Continue(())
+                },
+            )
+        };
         unsafe {
-            Self::trace_with_trap_state(vm_store_context, unwind, state, trap_pc_and_fp, |frame| {
-                frames.push(frame);
-                ControlFlow::Continue(())
-            });
+            Self::trace_with_trap_state(vm_store_context, state, trap_pc_and_fp, f);
         }
         Backtrace(frames)
     }
 
     /// Walk the current Wasm stack, calling `f` for each frame we walk.
     #[cfg(feature = "gc")]
-    pub fn trace(store: &StoreOpaque, f: impl FnMut(Frame) -> ControlFlow<()>) {
+    pub fn trace(store: &StoreOpaque, mut f: impl FnMut(Frame) -> ControlFlow<()>) {
         let vm_store_context = store.vm_store_context();
         let unwind = store.unwinder();
         tls::with(|state| match state {
             Some(state) => unsafe {
-                Self::trace_with_trap_state(vm_store_context, unwind, state, None, f)
+                let f = |activation: Activation| {
+                    wasmtime_unwinder::visit_frames(
+                        unwind,
+                        activation.exit_pc,
+                        activation.exit_fp,
+                        activation.entry_trampoline_fp,
+                        &mut f,
+                    )
+                };
+                Self::trace_with_trap_state(vm_store_context, state, None, f)
             },
             None => {}
         });
@@ -97,7 +123,7 @@ impl Backtrace {
     pub fn trace_suspended_continuation(
         store: &StoreOpaque,
         continuation: &VMContRef,
-        f: impl FnMut(Frame) -> ControlFlow<()>,
+        mut f: impl FnMut(Frame) -> ControlFlow<()>,
     ) {
         log::trace!("====== Capturing Backtrace (suspended continuation) ======");
 
@@ -122,9 +148,21 @@ impl Backtrace {
             let stack_chain =
                 VMStackChain::Continuation(continuation as *const VMContRef as *mut VMContRef);
 
-            if let ControlFlow::Break(()) =
-                Self::trace_through_continuations(unwind, stack_chain, pc, fp, trampoline_fp, f)
-            {
+            if let ControlFlow::Break(()) = Self::trace_through_continuations(
+                stack_chain,
+                pc,
+                fp,
+                trampoline_fp,
+                |activation| {
+                    wasmtime_unwinder::visit_frames(
+                        unwind,
+                        activation.exit_pc,
+                        activation.exit_fp,
+                        activation.entry_trampoline_fp,
+                        &mut f,
+                    )
+                },
+            ) {
                 log::trace!("====== Done Capturing Backtrace (closure break) ======");
                 return;
             }
@@ -153,10 +191,9 @@ impl Backtrace {
     /// other opaque host code; we don't know anything about it.
     pub(crate) unsafe fn trace_with_trap_state(
         vm_store_context: *const VMStoreContext,
-        unwind: &dyn Unwind,
         state: &CallThreadState,
         trap_pc_and_fp: Option<(usize, usize)>,
-        mut f: impl FnMut(Frame) -> ControlFlow<()>,
+        mut f: impl FnMut(Activation) -> ControlFlow<()>,
     ) {
         log::trace!("====== Capturing Backtrace ======");
 
@@ -219,9 +256,16 @@ impl Backtrace {
                 *pc != 0
             });
 
-        for (chain, pc, fp, sp) in activations {
-            let res =
-                unsafe { Self::trace_through_continuations(unwind, chain, pc, fp, sp, &mut f) };
+        for (chain, exit_pc, exit_fp, entry_trampoline_fp) in activations {
+            let res = unsafe {
+                Self::trace_through_continuations(
+                    chain,
+                    exit_pc,
+                    exit_fp,
+                    entry_trampoline_fp,
+                    &mut f,
+                )
+            };
             if let ControlFlow::Break(()) = res {
                 log::trace!("====== Done Capturing Backtrace (closure break) ======");
                 return;
@@ -240,20 +284,21 @@ impl Backtrace {
     /// ends with the initial stack. We then trace through each of these stacks
     /// individually, up to (and including) the initial stack.
     unsafe fn trace_through_continuations(
-        unwind: &dyn Unwind,
         chain: VMStackChain,
-        pc: usize,
-        fp: usize,
-        trampoline_fp: usize,
-        mut f: impl FnMut(Frame) -> ControlFlow<()>,
+        exit_pc: usize,
+        exit_fp: usize,
+        entry_trampoline_fp: usize,
+        mut f: impl FnMut(Activation) -> ControlFlow<()>,
     ) -> ControlFlow<()> {
         use crate::runtime::vm::stack_switching::{VMContRef, VMStackLimits};
 
         // Handle the stack that is currently running (which may be a
         // continuation or the initial stack).
-        unsafe {
-            wasmtime_unwinder::visit_frames(unwind, pc, fp, trampoline_fp, &mut f)?;
-        }
+        f(Activation {
+            exit_pc,
+            exit_fp,
+            entry_trampoline_fp,
+        })?;
 
         // Note that the rest of this function has no effect if `chain` is
         // `Some(VMStackChain::InitialStack(_))` (i.e., there is only one stack to
@@ -307,19 +352,33 @@ impl Backtrace {
                 debug_assert!(parent_stack_range.contains(&parent_limits.stack_limit));
             });
 
-            unsafe {
-                wasmtime_unwinder::visit_frames(
-                    unwind,
-                    resume_pc,
-                    resume_fp,
-                    parent_limits.last_wasm_entry_fp,
-                    &mut f,
-                )?
-            }
+            f(Activation {
+                exit_pc: resume_pc,
+                exit_fp: resume_fp,
+                entry_trampoline_fp: parent_limits.last_wasm_entry_fp,
+            })?;
         }
         ControlFlow::Continue(())
     }
 
+    /// Capture all Activations reachable from the current point
+    /// within a hostcall.
+    #[cfg(feature = "debug")]
+    fn activations(store: &StoreOpaque) -> Vec<Activation> {
+        let mut activations = vec![];
+        let vm_store_context = store.vm_store_context();
+        tls::with(|state| match state {
+            Some(state) => unsafe {
+                Self::trace_with_trap_state(vm_store_context, state, None, |act| {
+                    activations.push(act);
+                    ControlFlow::Continue(())
+                });
+            },
+            None => {}
+        });
+        activations
+    }
+
     /// Iterate over the frames inside this backtrace.
     pub fn frames<'a>(
         &'a self,
@@ -330,14 +389,14 @@ impl Backtrace {
 
 /// An iterator over one Wasm activation.
 #[cfg(feature = "debug")]
-pub(crate) struct CurrentActivationBacktrace<'a, T: 'static> {
+struct ActivationBacktrace<'a, T: 'static> {
     pub(crate) store: StoreContextMut<'a, T>,
     inner: Box<dyn Iterator<Item = Frame>>,
 }
 
 #[cfg(feature = "debug")]
-impl<'a, T: 'static> CurrentActivationBacktrace<'a, T> {
-    /// Return an iterator over the most recent Wasm activation.
+impl<'a, T: 'static> ActivationBacktrace<'a, T> {
+    /// Return an iterator over a Wasm activation.
     ///
     /// The iterator captures the store with a mutable borrow, and
     /// then yields it back at each frame. This ensures that the stack
@@ -348,43 +407,171 @@ impl<'a, T: 'static> CurrentActivationBacktrace<'a, T> {
     /// This serves as an alternative to `Backtrace::trace()` and
     /// friends: it allows external iteration (and e.g. lazily walking
     /// through frames in a stack) rather than visiting via a closure.
-    ///
-    /// # Safety
-    ///
-    /// Although the iterator provides mutable store as a public
-    /// field, this *must not* be used to mutate the stack activation
-    /// itself that this iterator is visiting. While the `store`
-    /// technically owns the stack in question, the only way to do
-    /// this with the current API would be to return back into the
-    /// Wasm activation. As long as this iterator is held and used
-    /// while within host code called from that activation (which will
-    /// ordinarily be ensured if the `store`'s lifetime came from the
-    /// host entry point) then everything will be sound.
-    pub(crate) unsafe fn new(store: StoreContextMut<'a, T>) -> CurrentActivationBacktrace<'a, T> {
-        // Get the initial exit FP, exit PC, and entry FP.
-        let vm_store_context = store.0.vm_store_context();
-        let exit_pc = unsafe { *(*vm_store_context).last_wasm_exit_pc.get() };
-        let exit_fp = unsafe { (*vm_store_context).last_wasm_exit_fp() };
-        let trampoline_fp = unsafe { *(*vm_store_context).last_wasm_entry_fp.get() };
-        let inner: Box<dyn Iterator<Item = Frame>> = if exit_fp == 0 {
+    pub(crate) fn new(
+        store: StoreContextMut<'a, T>,
+        activation: Activation,
+    ) -> ActivationBacktrace<'a, T> {
+        let inner: Box<dyn Iterator<Item = Frame>> = if activation.exit_fp == 0 {
             // No activations on this Store; return an empty iterator.
             Box::new(core::iter::empty())
         } else {
             let unwind = store.0.unwinder();
             // Establish the iterator.
             Box::new(unsafe {
-                wasmtime_unwinder::frame_iterator(unwind, exit_pc, exit_fp, trampoline_fp)
+                wasmtime_unwinder::frame_iterator(
+                    unwind,
+                    activation.exit_pc,
+                    activation.exit_fp,
+                    activation.entry_trampoline_fp,
+                )
             })
         };
 
-        CurrentActivationBacktrace { store, inner }
+        ActivationBacktrace { store, inner }
     }
 }
 
 #[cfg(feature = "debug")]
-impl<'a, T: 'static> Iterator for CurrentActivationBacktrace<'a, T> {
+impl<'a, T: 'static> Iterator for ActivationBacktrace<'a, T> {
     type Item = Frame;
     fn next(&mut self) -> Option<Self::Item> {
         self.inner.next()
     }
 }
+
+/// An iterator over all Wasm activations in a Store.
+#[cfg(feature = "debug")]
+pub(crate) struct StoreBacktrace<'a, T: 'static> {
+    /// The current activation iterator or the Store itself if no more
+    /// activations.
+    ///
+    /// This is `Option` so that we can move to the next while
+    /// transferring ownership of the Store without deconstructing
+    /// this whole iterator.
+    current: Option<StoreOrActivationBacktrace<'a, T>>,
+    activations: Vec<Activation>,
+}
+
+/// Either an iterator over a Wasm activation, or a `StoreContextMut`
+/// if no activations are left.
+#[cfg(feature = "debug")]
+enum StoreOrActivationBacktrace<'a, T: 'static> {
+    Store(StoreContextMut<'a, T>),
+    Activation(ActivationBacktrace<'a, T>),
+}
+
+#[cfg(feature = "debug")]
+impl<'a, T: 'static> StoreOrActivationBacktrace<'a, T> {
+    fn is_activation(&self) -> bool {
+        match self {
+            Self::Activation(_) => true,
+            _ => false,
+        }
+    }
+}
+
+#[cfg(feature = "debug")]
+impl<'a, T> StoreBacktrace<'a, T> {
+    /// Return an iterator over all Wasm activations in a Store, in
+    /// invocation order.
+    ///
+    /// The iterator captures the store with a mutable borrow, and
+    /// then yields it back at each frame. This ensures that the stack
+    /// remains live while still providing a mutable store that may be
+    /// needed to access items in the frame (e.g., to create new roots
+    /// when reading out GC refs).
+    ///
+    /// This serves as an alternative to `Backtrace::trace()` and
+    /// friends: it allows external iteration (and e.g. lazily walking
+    /// through frames in a stack) rather than visiting via a closure.
+    pub(crate) fn new(store: StoreContextMut<'a, T>) -> StoreBacktrace<'a, T> {
+        // Get all activations, in innermost-to-outermost order.
+        use crate::store::AsStoreOpaque;
+        let mut activations = Backtrace::activations(store.0.as_store_opaque());
+        // Reverse to outermost-to-innermost so we can pop off the end.
+        activations.reverse();
+        // Create our inner state: either an activation iterator on
+        // the innermost activation that owns the store, or a sentinel
+        // if there are no activations.
+        let current = match activations.pop() {
+            Some(innermost) => {
+                StoreOrActivationBacktrace::Activation(ActivationBacktrace::new(store, innermost))
+            }
+            None => StoreOrActivationBacktrace::Store(store),
+        };
+        StoreBacktrace {
+            current: Some(current),
+            activations,
+        }
+    }
+
+    /// Get the Store underlying this iteration.
+    pub fn store(&self) -> StoreContext<'_, T> {
+        match self.current.as_ref().unwrap() {
+            StoreOrActivationBacktrace::Activation(activation) => StoreContext(activation.store.0),
+            StoreOrActivationBacktrace::Store(store) => StoreContext(store.0),
+        }
+    }
+
+    /// Get the Store underlying this iteration.
+    pub fn store_mut(&mut self) -> StoreContextMut<'_, T> {
+        match self.current.as_mut().unwrap() {
+            StoreOrActivationBacktrace::Activation(activation) => {
+                StoreContextMut(activation.store.0)
+            }
+            StoreOrActivationBacktrace::Store(store) => StoreContextMut(store.0),
+        }
+    }
+
+    fn take_store(&mut self) -> StoreContextMut<'a, T> {
+        match self.current.take().unwrap() {
+            StoreOrActivationBacktrace::Activation(activation) => activation.store,
+            StoreOrActivationBacktrace::Store(store) => store,
+        }
+    }
+
+    /// Move to the next activation.
+    fn next_activation(&mut self) {
+        let activation = self.activations.pop();
+        let store = self.take_store();
+        self.current = Some(match activation {
+            Some(activation) => {
+                StoreOrActivationBacktrace::Activation(ActivationBacktrace::new(store, activation))
+            }
+            None => StoreOrActivationBacktrace::Store(store),
+        });
+    }
+}
+
+/// A single item in an iteration over a store's frames: either a Wasm
+/// frame, or a sentinel representing host code between activations.
+#[cfg(feature = "debug")]
+pub enum FrameOrHostCode {
+    /// A WebAsembly frame.
+    Frame(Frame),
+    /// Some number of host frames between Wasm activations.
+    HostCode,
+}
+
+#[cfg(feature = "debug")]
+impl<'a, T: 'static> Iterator for StoreBacktrace<'a, T> {
+    type Item = FrameOrHostCode;
+    fn next(&mut self) -> Option<Self::Item> {
+        match self.current.as_mut().unwrap() {
+            StoreOrActivationBacktrace::Store(_) => None,
+            StoreOrActivationBacktrace::Activation(act) => match act.next() {
+                Some(frame) => Some(FrameOrHostCode::Frame(frame)),
+                None => {
+                    self.next_activation();
+                    // If there's another activation waiting, return
+                    // HostCode between the two; otherwise, don't.
+                    if self.current.as_ref().unwrap().is_activation() {
+                        Some(FrameOrHostCode::HostCode)
+                    } else {
+                        None
+                    }
+                }
+            },
+        }
+    }
+}
