@@ -8,7 +8,7 @@ mod core;
 mod spectest;
 mod wast;
 
-pub use crate::spectest::{link_spectest, SpectestConfig};
+pub use crate::spectest::{SpectestConfig, link_spectest};
 pub use crate::wast::{Async, WastContext};
 
 /// Version number of this crate.
