@@ -1,17 +1,22 @@
 //! Common functionality shared between command implementations.
 
-use anyhow::{anyhow, bail, Context, Result};
+use anyhow::{Context, Result, anyhow, bail};
 use clap::Parser;
 use std::net::TcpListener;
 use std::{fs::File, path::Path, time::Duration};
 use wasmtime::{Engine, Module, Precompiled, StoreLimits, StoreLimitsBuilder};
-use wasmtime_cli_flags::{opt::WasmtimeOptionValue, CommonOptions};
-use wasmtime_wasi::bindings::LinkOptions;
+use wasmtime_cli_flags::{CommonOptions, opt::WasmtimeOptionValue};
 use wasmtime_wasi::WasiCtxBuilder;
 
 #[cfg(feature = "component-model")]
 use wasmtime::component::Component;
 
+/// Whether or not WASIp3 is enabled by default.
+///
+/// Currently this is disabled (the `&& false`), but that'll get removed in the
+/// future.
+pub const P3_DEFAULT: bool = cfg!(feature = "component-model-async") && false;
+
 pub enum RunTarget {
     Core(Module),
 
@@ -336,11 +341,60 @@ impl RunCommon {
         Ok(listeners)
     }
 
-    pub fn compute_wasi_features(&self) -> LinkOptions {
-        let mut options = LinkOptions::default();
-        options.cli_exit_with_code(self.common.wasi.cli_exit_with_code.unwrap_or(false));
-        options.network_error_code(self.common.wasi.network_error_code.unwrap_or(false));
-        options
+    pub fn validate_p3_option(&self) -> Result<()> {
+        let p3 = self.common.wasi.p3.unwrap_or(P3_DEFAULT);
+        if p3 && !cfg!(feature = "component-model-async") {
+            bail!("support for WASIp3 disabled at compile time");
+        }
+        Ok(())
+    }
+
+    pub fn validate_cli_enabled(&self) -> Result<Option<bool>> {
+        let mut cli = self.common.wasi.cli;
+
+        // Accept -Scommon as a deprecated alias for -Scli.
+        if let Some(common) = self.common.wasi.common {
+            if cli.is_some() {
+                bail!(
+                    "The -Scommon option should not be use with -Scli as it is a deprecated alias"
+                );
+            } else {
+                // In the future, we may add a warning here to tell users to use
+                // `-S cli` instead of `-S common`.
+                cli = Some(common);
+            }
+        }
+
+        Ok(cli)
+    }
+
+    /// Adds `wasmtime-wasi` interfaces (dubbed "-Scli" in the flags to the
+    /// `wasmtime` command) to the `linker` provided.
+    ///
+    /// This will handle adding various WASI standard versions to the linker
+    /// internally.
+    #[cfg(feature = "component-model")]
+    pub fn add_wasmtime_wasi_to_linker<T>(
+        &self,
+        linker: &mut wasmtime::component::Linker<T>,
+    ) -> Result<()>
+    where
+        T: wasmtime_wasi::WasiView,
+    {
+        let mut p2_options = wasmtime_wasi::p2::bindings::LinkOptions::default();
+        p2_options.cli_exit_with_code(self.common.wasi.cli_exit_with_code.unwrap_or(false));
+        p2_options.network_error_code(self.common.wasi.network_error_code.unwrap_or(false));
+        wasmtime_wasi::p2::add_to_linker_with_options_async(linker, &p2_options)?;
+
+        #[cfg(feature = "component-model-async")]
+        if self.common.wasi.p3.unwrap_or(P3_DEFAULT) {
+            let mut p3_options = wasmtime_wasi::p3::bindings::LinkOptions::default();
+            p3_options.cli_exit_with_code(self.common.wasi.cli_exit_with_code.unwrap_or(false));
+            wasmtime_wasi::p3::add_to_linker_with_options(linker, &p3_options)
+                .context("failed to link `wasi:cli@0.3.x`")?;
+        }
+
+        Ok(())
     }
 }
 
@@ -375,3 +429,9 @@ impl Profile {
         }
     }
 }
+
+#[derive(Default, Clone)]
+#[cfg(all(feature = "wasi-http", feature = "component-model-async"))]
+pub struct DefaultP3Ctx;
+#[cfg(all(feature = "wasi-http", feature = "component-model-async"))]
+impl wasmtime_wasi_http::p3::WasiHttpCtx for DefaultP3Ctx {}
