@@ -2165,6 +2165,23 @@ start a print 1234
         Ok(())
     }
 
+    #[test]
+    fn p2_cli_p3_hello_stdout_post_return() -> Result<()> {
+        let output = run_wasmtime(&[
+            "run",
+            "-Wcomponent-model-async",
+            "-Sp3",
+            P3_CLI_HELLO_STDOUT_POST_RETURN_COMPONENT,
+        ]);
+        if cfg!(feature = "component-model-async") {
+            let output = output?;
+            assert_eq!(output, "hello, world\nhello again, after return\n");
+        } else {
+            assert!(output.is_err());
+        }
+        Ok(())
+    }
+
     mod invoke {
         use super::*;
 
@@ -2523,13 +2540,14 @@ fn profile_with_vtune() -> Result<()> {
     println!("> executing: {bin:?}");
     let output = bin.output()?;
 
-    assert!(output.status.success());
     let stdout = String::from_utf8_lossy(&output.stdout);
     let stderr = String::from_utf8_lossy(&output.stderr);
     println!("> stdout:\n{stdout}");
-    assert!(stdout.contains("CPU Time"));
     println!("> stderr:\n{stderr}");
+
+    assert!(output.status.success());
     assert!(!stderr.contains("Error"));
+    assert!(stdout.contains("CPU Time"));
     Ok(())
 }
 
