@@ -39,9 +39,9 @@
 //! at the start of the file. These comments are then parsed as TOML and
 //! deserialized into `TestConfig` in this crate.
 
-use anyhow::{bail, Context, Result};
+use anyhow::{Context, Result, bail};
 use clap::Parser;
-use cranelift_codegen::ir::{Function, UserExternalName, UserFuncName};
+use cranelift_codegen::ir::Function;
 use libtest_mimic::{Arguments, Trial};
 use serde_derive::Deserialize;
 use similar::TextDiff;
@@ -50,11 +50,22 @@ use std::io::Write as _;
 use std::path::{Path, PathBuf};
 use std::process::Stdio;
 use tempfile::TempDir;
-use wasmtime::{Engine, OptLevel, Strategy};
+use wasmtime::{CodeBuilder, CodeHint, Engine, OptLevel, Strategy};
 use wasmtime_cli_flags::CommonOptions;
 
 fn main() -> Result<()> {
-    if cfg!(miri) {
+    if cfg!(miri) || cfg!(asan) {
+        return Ok(());
+    }
+
+    // There's not a ton of use in emulating these tests on other architectures
+    // since they only exercise architecture-independent code of compiling to
+    // multiple architectures. Additionally CI seems to occasionally deadlock or
+    // get stuck in these tests when using QEMU, and it's not entirely clear
+    // why. Finally QEMU-emulating these tests is relatively slow and without
+    // much benefit from emulation it's hard to justify this. In the end disable
+    // this test suite when QEMU is enabled.
+    if std::env::var("WASMTIME_TEST_NO_HOG_MEMORY").is_ok() {
         return Ok(());
     }
 
@@ -112,6 +123,8 @@ struct TestConfig {
     test: TestKind,
     flags: Option<TestConfigFlags>,
     objdump: Option<TestConfigFlags>,
+    filter: Option<String>,
+    unsafe_intrinsics: Option<String>,
 }
 
 #[derive(Debug, Deserialize)]
@@ -158,13 +171,14 @@ impl Test {
     fn new(path: &Path) -> Result<Test> {
         let contents =
             std::fs::read_to_string(path).with_context(|| format!("failed to read {path:?}"))?;
-        let config: TestConfig = wasmtime_test_util::wast::parse_test_config(&contents)
+        let config: TestConfig = wasmtime_test_util::wast::parse_test_config(&contents, ";;!")
             .context("failed to parse test configuration as TOML")?;
         let mut flags = vec!["wasmtime"];
         if let Some(config) = &config.flags {
             flags.extend(config.to_vec());
         }
-        let opts = wasmtime_cli_flags::CommonOptions::try_parse_from(&flags)?;
+        let mut opts = wasmtime_cli_flags::CommonOptions::try_parse_from(&flags)?;
+        opts.codegen.cranelift_debug_verifier = Some(true);
 
         Ok(Test {
             path: path.to_path_buf(),
@@ -195,25 +209,49 @@ impl Test {
             }
         }
         let engine = Engine::new(&config).context("failed to create engine")?;
-        let module = wat::parse_file(&self.path)?;
-        let elf = engine
-            .precompile_module(&module)
-            .context("failed to compile module")?;
+
+        let mut builder = CodeBuilder::new(&engine);
+        builder.wasm_binary_or_text_file(&self.path)?;
+        if let Some(name) = self.config.unsafe_intrinsics.as_deref() {
+            unsafe {
+                builder.expose_unsafe_intrinsics(name);
+            }
+        }
+
+        let elf = match builder.hint() {
+            Some(CodeHint::Component) => builder
+                .compile_component_serialized()
+                .context("failed to compile component")?,
+            Some(CodeHint::Module) => builder
+                .compile_module_serialized()
+                .context("failed to compile module")?,
+            None => bail!(
+                "contents of `{}` do not look like a Wasm component or module",
+                self.path.display()
+            ),
+        };
 
         match self.config.test {
             TestKind::Clif | TestKind::Optimize => {
                 // Read all `*.clif` files from the clif directory that the
                 // compilation process just emitted.
                 let mut clifs = Vec::new();
-                for entry in tempdir
+
+                // Sort entries for determinism; multiple wasm modules can
+                // generate clif functions with the same names, so sorting the
+                // resulting clif functions alone isn't good enough.
+                let mut entries = tempdir
                     .path()
                     .read_dir()
                     .context("failed to read tempdir")?
-                {
-                    let entry = entry.context("failed to iterate over tempdir")?;
-                    let path = entry.path();
+                    .map(|e| Ok(e.context("failed to iterate over tempdir")?.path()))
+                    .collect::<Result<Vec<_>>>()?;
+                entries.sort();
+
+                for path in entries {
                     if let Some(name) = path.file_name().and_then(|s| s.to_str()) {
-                        if !name.starts_with("wasm_func_") {
+                        let filter = self.config.filter.as_deref().unwrap_or("wasm[0]--function");
+                        if !name.contains(filter) {
                             continue;
                         }
                     }
@@ -224,20 +262,20 @@ impl Test {
 
                 // Parse the text format CLIF which is emitted by Wasmtime back
                 // into in-memory data structures.
-                let mut functions = clifs
+                let functions = clifs
                     .iter()
                     .map(|clif| {
-                        let mut funcs = cranelift_reader::parse_functions(clif)?;
+                        let mut funcs =
+                            cranelift_reader::parse_functions(clif).with_context(|| {
+                                format!("failed to parse CLIF:\n\"\"\"\n{clif}\n\"\"\"")
+                            })?;
                         if funcs.len() != 1 {
                             bail!("expected one function per clif");
                         }
                         Ok(funcs.remove(0))
                     })
                     .collect::<Result<Vec<_>>>()?;
-                functions.sort_by_key(|f| match f.name {
-                    UserFuncName::User(UserExternalName { namespace, index }) => (namespace, index),
-                    UserFuncName::Testcase(_) => unreachable!(),
-                });
+
                 Ok(CompileOutput::Clif(functions))
             }
             TestKind::Compile | TestKind::Winch => Ok(CompileOutput::Elf(elf)),
@@ -276,6 +314,9 @@ fn assert_output(test: &Test, output: CompileOutput) -> Result<()> {
                     cmd.arg("--traps=false");
                 }
             }
+            if let Some(filter) = &test.config.filter {
+                cmd.arg("--filter").arg(filter);
+            }
 
             let mut child = cmd.spawn().context("failed to run wasmtime")?;
             child
