@@ -16,7 +16,6 @@ use wit_component::ComponentEncoder;
 
 const KNOWN_FAILURES: &[&str] = &[
     "filesystem-hard-links",
-    "http-response",
     "filesystem-read-directory",
     // FIXME(#11524)
     "remove_directory_trailing_slashes",
@@ -64,8 +63,6 @@ const KNOWN_FAILURES: &[&str] = &[
     #[cfg(windows)]
     "path_open_missing",
     #[cfg(windows)]
-    "path_open_preopen",
-    #[cfg(windows)]
     "path_open_read_write",
     #[cfg(windows)]
     "path_rename",
@@ -96,6 +93,10 @@ const KNOWN_FAILURES: &[&str] = &[
     "multi-clock-wait",
     "monotonic-clock",
     "filesystem-advise",
+    // Wasmtime's snapshot of WASIp3 APIs is different than what these tests are
+    // expecting.
+    "wall-clock",
+    "http-response",
 ];
 
 fn main() -> Result<()> {
